"""特許・意匠・商標分類の列挙型モジュール。

特許種別、特許 FI 分類、意匠日本分類、商標類似商品・役務審査基準を定義する。
"""

from __future__ import annotations

from enum import StrEnum


class PatentType(StrEnum):
    """特許種別の列挙型。

    ``PatentInfoV2.patent_type`` フィールドの値に対応する。
    特許・意匠・商標の3種別を定義する。
    """

    特許 = "特許"
    意匠 = "意匠"
    商標 = "商標"


# PatentType 英語 alias
PATENT = PatentType.特許
DESIGN = PatentType.意匠
TRADEMARK = PatentType.商標


class PatentClassification(StrEnum):
    """特許 FI 分類の列挙型。

    IPC（国際特許分類）に基づく FI 分類のセクション・クラスを定義する。
    ``description_ja`` / ``description_en`` プロパティで日英の分類名を取得できる。
    """

    農業_林業_畜産 = "A01"
    ベイキング_生地製造 = "A21"
    屠殺_肉処理 = "A22"
    食品_食料品 = "A23"
    たばこ_喫煙具 = "A24"
    衣類 = "A41"
    頭部着用物 = "A42"
    履物 = "A43"
    小間物_貴金属宝石 = "A44"
    手持品_旅行用品 = "A45"
    ブラシ製品 = "A46"
    家具_家庭用品 = "A47"
    医学_獣医学_衛生学 = "A61"
    人命救助_消防 = "A62"
    スポーツ_ゲーム_娯楽 = "A63"
    セクションA_その他 = "A99"
    物理的化学的方法_装置 = "B01"
    破砕_粉砕 = "B02"
    固体物質の分離 = "B03"
    遠心装置 = "B04"
    霧化_噴霧 = "B05"
    機械的振動 = "B06"
    固体分離_仕分け = "B07"
    清掃 = "B08"
    固体廃棄物処理 = "B09"
    機械的金属加工 = "B21"
    鋳造_粉末冶金 = "B22"
    工作機械_金属加工 = "B23"
    研削_研磨 = "B24"
    手工具_動力工具 = "B25"
    切断手工具_切断機 = "B26"
    木材加工 = "B27"
    セメント_粘土_石材加工 = "B28"
    プラスチック加工 = "B29"
    プレス = "B30"
    紙_板紙製品の製造 = "B31"
    積層体 = "B32"
    付加製造技術 = "B33"
    印刷_タイプライター = "B41"
    製本_アルバム = "B42"
    筆記_製図用器具 = "B43"
    装飾技術 = "B44"
    車両一般 = "B60"
    鉄道 = "B61"
    路面車両 = "B62"
    船舶_水上構造物 = "B63"
    航空機_宇宙工学 = "B64"
    運搬_包装_貯蔵 = "B65"
    巻上装置_揚重装置 = "B66"
    容器開封密封_液体取扱い = "B67"
    馬具_詰め物 = "B68"
    マイクロ構造技術 = "B81"
    ナノテクノロジー = "B82"
    セクションB_その他 = "B99"
    無機化学 = "C01"
    水_廃水処理 = "C02"
    ガラス_鉱物 = "C03"
    セメント_コンクリート_セラミックス = "C04"
    肥料 = "C05"
    火薬_マッチ = "C06"
    有機化学 = "C07"
    有機高分子化合物 = "C08"
    染料_ペイント_接着剤 = "C09"
    石油_ガス_コークス工業 = "C10"
    油脂_洗浄剤 = "C11"
    生化学_微生物学 = "C12"
    糖工業 = "C13"
    皮革 = "C14"
    鉄冶金 = "C21"
    冶金_合金 = "C22"
    金属被覆_表面処理 = "C23"
    電気分解_電気泳動 = "C25"
    結晶成長 = "C30"
    コンビナトリアル技術 = "C40"
    セクションC_その他 = "C99"
    糸_繊維_紡績 = "D01"
    糸_ロープ仕上げ = "D02"
    織成 = "D03"
    組みひも_メリヤス編成 = "D04"
    縫製_刺しゅう = "D05"
    繊維処理_洗濯 = "D06"
    ロープ_ケーブル = "D07"
    製紙_セルロース = "D21"
    セクションD_その他 = "D99"
    道路_鉄道_橋りょう建設 = "E01"
    水工_基礎_土砂移送 = "E02"
    上水_下水 = "E03"
    建築物 = "E04"
    錠_鍵_金庫 = "E05"
    戸_窓_シャッタ = "E06"
    削孔_採鉱 = "E21"
    セクションE_その他 = "E99"
    機関一般_蒸気機関 = "F01"
    燃焼機関 = "F02"
    液体用機関_風力原動機 = "F03"
    容積形機械_ポンプ = "F04"
    流体圧アクチュエータ = "F15"
    機械要素 = "F16"
    ガス_液体の貯蔵分配 = "F17"
    照明 = "F21"
    蒸気発生 = "F22"
    燃焼装置 = "F23"
    加熱_レンジ_換気 = "F24"
    冷凍_冷却_ヒートポンプ = "F25"
    乾燥 = "F26"
    炉_キルン_レトルト = "F27"
    熱交換 = "F28"
    武器 = "F41"
    弾薬_爆破 = "F42"
    セクションF_その他 = "F99"
    測定_試験 = "G01"
    光学 = "G02"
    写真_映画_ホログラフイ = "G03"
    時計 = "G04"
    制御_調整 = "G05"
    計算_計数 = "G06"
    チェック装置 = "G07"
    信号 = "G08"
    教育_暗号方法_表示_広告 = "G09"
    楽器_音響 = "G10"
    情報記憶 = "G11"
    器械の細部 = "G12"
    情報通信技術_ICT = "G16"
    核物理_核工学 = "G21"
    セクションG_その他 = "G99"
    基本的電気素子 = "H01"
    電力の発電_変換_配電 = "H02"
    基本電子回路 = "H03"
    電気通信技術 = "H04"
    電気技術 = "H05"
    セクションH_その他 = "H99"

    @property
    def description_ja(self) -> str:
        """日本語の分類名を返す。

        Returns:
            日本語の分類名文字列。
        """
        return _PATENT_DESC_JA[self]

    @property
    def description_en(self) -> str:
        """英語の分類名を返す。

        Returns:
            英語の分類名文字列。
        """
        return _PATENT_DESC_EN[self]


_PATENT_DESC_JA: dict[PatentClassification, str] = {
    PatentClassification.農業_林業_畜産: "農業; 林業; 畜産; 狩猟; 捕獲; 漁業",
    PatentClassification.ベイキング_生地製造: "ベイキング; 生地製造または加工の機械あるいは設備; ベイキングの生地",
    PatentClassification.屠殺_肉処理: "屠殺; 肉処理; 家禽または魚の処理",
    PatentClassification.食品_食料品: "食品または食料品; 他のクラスに包含されないそれらの処理",
    PatentClassification.たばこ_喫煙具: "たばこ; 葉巻たばこ; 紙巻たばこ; 喫煙具",
    PatentClassification.衣類: "衣類",
    PatentClassification.頭部着用物: "頭部に着用するもの",
    PatentClassification.履物: "履物",
    PatentClassification.小間物_貴金属宝石: "小間物; 貴金属宝石類",
    PatentClassification.手持品_旅行用品: "手持品または旅行用品",
    PatentClassification.ブラシ製品: "ブラシ製品",
    PatentClassification.家具_家庭用品: "家具; 家庭用品または家庭用設備; コーヒーひき; 香辛料ひき; 真空掃除機一般",
    PatentClassification.医学_獣医学_衛生学: "医学または獣医学; 衛生学",
    PatentClassification.人命救助_消防: "人命救助; 消防",
    PatentClassification.スポーツ_ゲーム_娯楽: "スポーツ; ゲーム; 娯楽",
    PatentClassification.セクションA_その他: "このセクションの中で他に分類されない主題事項",
    PatentClassification.物理的化学的方法_装置: "物理的または化学的方法または装置一般",
    PatentClassification.破砕_粉砕: "破砕, または粉砕; 製粉のための穀粒の前処理",
    PatentClassification.固体物質の分離: "液体による, または, 風力テーブルまたはジグによる固体物質の分離; 固体物質または流体から固体物質の磁気または静電気による分離, 高圧電界による分離",
    PatentClassification.遠心装置: "物理的または化学的工程を行なうための遠心装置または機械",
    PatentClassification.霧化_噴霧: "霧化または噴霧一般; 液体または他の流動性材料の表面への適用一般",
    PatentClassification.機械的振動: "機械的振動の発生または伝達一般",
    PatentClassification.固体分離_仕分け: "固体相互の分離; 仕分け",
    PatentClassification.清掃: "清掃",
    PatentClassification.固体廃棄物処理: "固体廃棄物の処理; 汚染土壌の再生",
    PatentClassification.機械的金属加工: "本質的には材料の除去が行なわれない機械的金属加工; 金属の打抜き",
    PatentClassification.鋳造_粉末冶金: "鋳造; 粉末冶金",
    PatentClassification.工作機械_金属加工: "工作機械; 他に分類されない金属加工",
    PatentClassification.研削_研磨: "研削; 研磨",
    PatentClassification.手工具_動力工具: "手工具; 可搬型動力工具; 手工具用の柄; 作業場設備; マニプレータ",
    PatentClassification.切断手工具_切断機: "切断手工具; 切断; 切断機",
    PatentClassification.木材加工: "木材または類似の材料の加工または保存; 釘打ち機またはステープル打ち機一般",
    PatentClassification.セメント_粘土_石材加工: "セメント, 粘土, または石材の加工",
    PatentClassification.プラスチック加工: "プラスチックの加工; 可塑状態の物質の加工一般",
    PatentClassification.プレス: "プレス",
    PatentClassification.紙_板紙製品の製造: "紙，板紙または紙と同様の方法で加工される材料からなる物品の製造；紙，板紙または紙と同様の方法で加工される材料の加工",
    PatentClassification.積層体: "積層体",
    PatentClassification.付加製造技術: "付加製造技術",
    PatentClassification.印刷_タイプライター: "印刷; 線画機; タイプライター; スタンプ",
    PatentClassification.製本_アルバム: "製本; アルバム; ファイル; 特殊印刷物",
    PatentClassification.筆記_製図用器具: "筆記用または製図用の器具; 机上付属具",
    PatentClassification.装飾技術: "装飾技術",
    PatentClassification.車両一般: "車両一般",
    PatentClassification.鉄道: "鉄道",
    PatentClassification.路面車両: "鉄道以外の路面車両",
    PatentClassification.船舶_水上構造物: "船舶またはその他の水上浮揚構造物; 関連艤装品",
    PatentClassification.航空機_宇宙工学: "航空機; 飛行; 宇宙工学",
    PatentClassification.運搬_包装_貯蔵: "運搬; 包装; 貯蔵; 薄板状または線条材料の取扱い",
    PatentClassification.巻上装置_揚重装置: "巻上装置; 揚重装置; 牽引装置",
    PatentClassification.容器開封密封_液体取扱い: "びん, 広口びんまたは類似の容器の開封または密封; 液体の取扱い",
    PatentClassification.馬具_詰め物: "馬具; 詰め物, かわ張りされた物品",
    PatentClassification.マイクロ構造技術: "マイクロ構造技術",
    PatentClassification.ナノテクノロジー: "ナノテクノロジー",
    PatentClassification.セクションB_その他: "このセクションの中で他に分類されない主題事項",
    PatentClassification.無機化学: "無機化学",
    PatentClassification.水_廃水処理: "水, 廃水, 下水または汚泥の処理",
    PatentClassification.ガラス_鉱物: "ガラス; 鉱物またはスラグウール",
    PatentClassification.セメント_コンクリート_セラミックス: "セメント; コンクリート; 人造石; セラミックス; 耐火物",
    PatentClassification.肥料: "肥料; 肥料の製造",
    PatentClassification.火薬_マッチ: "火薬; マッチ",
    PatentClassification.有機化学: "有機化学",
    PatentClassification.有機高分子化合物: "有機高分子化合物; その製造または化学的加工; それに基づく組成物",
    PatentClassification.染料_ペイント_接着剤: "染料; ペイント; つや出し剤; 天然樹脂; 接着剤; 他に分類されない組成物; 他に分類されない材料の応用",
    PatentClassification.石油_ガス_コークス工業: "石油, ガスまたはコークス工業; 一酸化炭素を含有する工業ガス; 燃料; 潤滑剤; でい炭",
    PatentClassification.油脂_洗浄剤: "動物性または植物性油, 脂肪, 脂肪性物質またはろう; それに由来する脂肪酸; 洗浄剤; ろうそく",
    PatentClassification.生化学_微生物学: "生化学; ビール; 酒精; ぶどう酒; 酢; 微生物学; 酵素学; 突然変異または遺伝子工学",
    PatentClassification.糖工業: "糖工業",
    PatentClassification.皮革: "原皮; 裸皮; 生皮; なめし革",
    PatentClassification.鉄冶金: "鉄冶金",
    PatentClassification.冶金_合金: "冶金; 鉄または非鉄合金; 合金の処理または非鉄金属の処理",
    PatentClassification.金属被覆_表面処理: "金属質材料への被覆; 金属質材料による材料への被覆; 化学的表面処理; 金属質材料の拡散処理; 真空蒸着, スパッタリング, イオン注入法, または化学蒸着による被覆一般; 金属質材料の防食または鉱皮の抑制一般",
    PatentClassification.電気分解_電気泳動: "電気分解または電気泳動方法; そのための装置",
    PatentClassification.結晶成長: "結晶成長",
    PatentClassification.コンビナトリアル技術: "コンビナトリアル技術",
    PatentClassification.セクションC_その他: "このセクションの中で他に分類されない主題事項",
    PatentClassification.糸_繊維_紡績: "天然または人造の糸または繊維; 紡績",
    PatentClassification.糸_ロープ仕上げ: "糸; 糸またはロープの機械的な仕上げ; 整経またはビーム巻き取り",
    PatentClassification.織成: "織成",
    PatentClassification.組みひも_メリヤス編成: "組みひも; レース編み; メリヤス編成; 縁とり; 不織布",
    PatentClassification.縫製_刺しゅう: "縫製; 刺しゅう; タフティング",
    PatentClassification.繊維処理_洗濯: "繊維または類似のものの処理; 洗濯; 他に分類されない可とう性材料",
    PatentClassification.ロープ_ケーブル: "ロープ; 電気的なもの以外のケーブル",
    PatentClassification.製紙_セルロース: "製紙; セルロースの製造",
    PatentClassification.セクションD_その他: "このセクションの中で他に分類されない主題事項",
    PatentClassification.道路_鉄道_橋りょう建設: "道路, 鉄道または橋りょうの建設",
    PatentClassification.水工_基礎_土砂移送: "水工; 基礎; 土砂の移送",
    PatentClassification.上水_下水: "上水; 下水",
    PatentClassification.建築物: "建築物",
    PatentClassification.錠_鍵_金庫: "錠; 鍵（かぎ）; 窓または戸の付属品; 金庫",
    PatentClassification.戸_窓_シャッタ: "戸, 窓, シャッタまたはローラブラインド一般; はしご",
    PatentClassification.削孔_採鉱: "地中もしくは岩石の削孔; 採鉱",
    PatentClassification.セクションE_その他: "このセクションの中で他に分類されない主題事項",
    PatentClassification.機関一般_蒸気機関: "機械または機関一般; 機関設備一般; 蒸気機関",
    PatentClassification.燃焼機関: "燃焼機関; 熱ガスまたは燃焼生成物を利用する機関設備",
    PatentClassification.液体用機関_風力原動機: "液体用機械または機関; 風力原動機, ばね原動機, 重力原動機; 他類に属さない機械動力または反動推進力を発生するもの",
    PatentClassification.容積形機械_ポンプ: "液体用容積形機械; 液体または圧縮性流体用ポンプ",
    PatentClassification.流体圧アクチュエータ: "流体圧アクチュエータ; 水力学または空気力学一般",
    PatentClassification.機械要素: "機械要素または単位; 機械または装置の効果的機能を生じ維持するための一般的手段",
    PatentClassification.ガス_液体の貯蔵分配: "ガスまたは液体の貯蔵または分配",
    PatentClassification.照明: "照明",
    PatentClassification.蒸気発生: "蒸気発生",
    PatentClassification.燃焼装置: "燃焼装置; 燃焼方法",
    PatentClassification.加熱_レンジ_換気: "加熱; レンジ; 換気",
    PatentClassification.冷凍_冷却_ヒートポンプ: "冷凍または冷却; 加熱と冷凍との組み合わせシステム; ヒートポンプシステム; 氷の製造または貯蔵; 気体の液化または固体化",
    PatentClassification.乾燥: "乾燥",
    PatentClassification.炉_キルン_レトルト: "炉, キルン, 窯（かま）; レトルト",
    PatentClassification.熱交換: "熱交換一般",
    PatentClassification.武器: "武器",
    PatentClassification.弾薬_爆破: "弾薬; 爆破",
    PatentClassification.セクションF_その他: "このセクションの中で他に分類されない主題事項",
    PatentClassification.測定_試験: "測定; 試験",
    PatentClassification.光学: "光学",
    PatentClassification.写真_映画_ホログラフイ: "写真; 映画; 光波以外の波を使用する類似技術; 電子写真; ホログラフイ",
    PatentClassification.時計: "時計",
    PatentClassification.制御_調整: "制御; 調整",
    PatentClassification.計算_計数: "計算; 計数",
    PatentClassification.チェック装置: "チェック装置",
    PatentClassification.信号: "信号",
    PatentClassification.教育_暗号方法_表示_広告: "教育; 暗号方法; 表示; 広告; シール",
    PatentClassification.楽器_音響: "楽器; 音響",
    PatentClassification.情報記憶: "情報記憶",
    PatentClassification.器械の細部: "器械の細部",
    PatentClassification.情報通信技術_ICT: "特定の用途分野に特に適合した情報通信技術［ＩＣＴ］",
    PatentClassification.核物理_核工学: "核物理; 核工学",
    PatentClassification.セクションG_その他: "このセクションの中で他に分類されない主題事項",
    PatentClassification.基本的電気素子: "基本的電気素子",
    PatentClassification.電力の発電_変換_配電: "電力の発電, 変換, 配電",
    PatentClassification.基本電子回路: "基本電子回路",
    PatentClassification.電気通信技術: "電気通信技術",
    PatentClassification.電気技術: "他に分類されない電気技術",
    PatentClassification.セクションH_その他: "このセクションの中で他に分類されない主題事項",
}

_PATENT_DESC_EN: dict[PatentClassification, str] = {
    PatentClassification.農業_林業_畜産: "Agriculture; Forestry; Animal husbandry; Hunting; Trapping; Fishing",
    PatentClassification.ベイキング_生地製造: "Baking; Edible doughs",
    PatentClassification.屠殺_肉処理: "Butchering; Meat treatment; Processing poultry or fish",
    PatentClassification.食品_食料品: "Foods or foodstuffs; Treatment thereof",
    PatentClassification.たばこ_喫煙具: "Tobacco; Cigars; Cigarettes; Smokers' requisites",
    PatentClassification.衣類: "Wearing apparel",
    PatentClassification.頭部着用物: "Headwear",
    PatentClassification.履物: "Footwear",
    PatentClassification.小間物_貴金属宝石: "Haberdashery; Jewellery",
    PatentClassification.手持品_旅行用品: "Hand or travelling articles",
    PatentClassification.ブラシ製品: "Brushware",
    PatentClassification.家具_家庭用品: "Furniture; Domestic articles or appliances; Coffee mills; Spice mills; Suction cleaners in general",
    PatentClassification.医学_獣医学_衛生学: "Medical or veterinary science; Hygiene",
    PatentClassification.人命救助_消防: "Life-saving; Fire-fighting",
    PatentClassification.スポーツ_ゲーム_娯楽: "Sports; Games; Amusements",
    PatentClassification.セクションA_その他: "Subject matter not otherwise provided for in this section",
    PatentClassification.物理的化学的方法_装置: "Physical or chemical processes or apparatus in general",
    PatentClassification.破砕_粉砕: "Crushing, pulverising, or disintegrating; Preparatory treatment of grain for milling",
    PatentClassification.固体物質の分離: "Separation of solid materials using liquids or using pneumatic tables or jigs",
    PatentClassification.遠心装置: "Centrifugal apparatus or machines for carrying-out physical or chemical processes",
    PatentClassification.霧化_噴霧: "Spraying or atomising in general; Applying fluent materials to surfaces, in general",
    PatentClassification.機械的振動: "Generating or transmitting mechanical vibrations in general",
    PatentClassification.固体分離_仕分け: "Separating solids from solids; Sorting",
    PatentClassification.清掃: "Cleaning",
    PatentClassification.固体廃棄物処理: "Disposal of solid waste; Reclamation of contaminated soil",
    PatentClassification.機械的金属加工: "Mechanical metal-working without essentially removing material; Punching metal",
    PatentClassification.鋳造_粉末冶金: "Casting; Powder metallurgy",
    PatentClassification.工作機械_金属加工: "Machine tools; Metal-working not otherwise provided for",
    PatentClassification.研削_研磨: "Grinding; Polishing",
    PatentClassification.手工具_動力工具: "Hand tools; Portable power-driven tools; Manipulators",
    PatentClassification.切断手工具_切断機: "Hand cutting tools; Cutting; Severing",
    PatentClassification.木材加工: "Working or preserving wood or similar material",
    PatentClassification.セメント_粘土_石材加工: "Working cement, clay, or stone",
    PatentClassification.プラスチック加工: "Working of plastics; Working of substances in a plastic state in general",
    PatentClassification.プレス: "Presses",
    PatentClassification.紙_板紙製品の製造: "Making articles of paper, cardboard or material worked in a manner analogous to paper",
    PatentClassification.積層体: "Layered products",
    PatentClassification.付加製造技術: "Additive manufacturing technology",
    PatentClassification.印刷_タイプライター: "Printing; Lining machines; Typewriters; Stamps",
    PatentClassification.製本_アルバム: "Bookbinding; Albums; Files; Special printed matter",
    PatentClassification.筆記_製図用器具: "Writing or drawing implements; Bureau accessories",
    PatentClassification.装飾技術: "Decorative arts",
    PatentClassification.車両一般: "Vehicles in general",
    PatentClassification.鉄道: "Railways",
    PatentClassification.路面車両: "Land vehicles for travelling otherwise than on rails",
    PatentClassification.船舶_水上構造物: "Ships or other waterborne vessels; Related equipment",
    PatentClassification.航空機_宇宙工学: "Aircraft; Aviation; Cosmonautics",
    PatentClassification.運搬_包装_貯蔵: "Conveying; Packing; Storing; Handling thin or filamentary material",
    PatentClassification.巻上装置_揚重装置: "Hoisting; Lifting; Hauling",
    PatentClassification.容器開封密封_液体取扱い: "Opening or closing bottles, jars or similar containers; Liquid handling",
    PatentClassification.馬具_詰め物: "Saddlery; Upholstery",
    PatentClassification.マイクロ構造技術: "Microstructural technology",
    PatentClassification.ナノテクノロジー: "Nanotechnology",
    PatentClassification.セクションB_その他: "Subject matter not otherwise provided for in this section",
    PatentClassification.無機化学: "Inorganic chemistry",
    PatentClassification.水_廃水処理: "Treatment of water, waste water, sewage, or sludge",
    PatentClassification.ガラス_鉱物: "Glass; Mineral or slag wool",
    PatentClassification.セメント_コンクリート_セラミックス: "Cements; Concrete; Artificial stone; Ceramics; Refractories",
    PatentClassification.肥料: "Fertilisers; Manufacture thereof",
    PatentClassification.火薬_マッチ: "Explosives; Matches",
    PatentClassification.有機化学: "Organic chemistry",
    PatentClassification.有機高分子化合物: "Organic macromolecular compounds",
    PatentClassification.染料_ペイント_接着剤: "Dyes; Paints; Polishes; Natural resins; Adhesives",
    PatentClassification.石油_ガス_コークス工業: "Petroleum, gas or coke industries; Technical gases containing carbon monoxide; Fuels; Lubricants; Peat",
    PatentClassification.油脂_洗浄剤: "Animal or vegetable oils, fats, fatty substances or waxes; Detergents; Candles",
    PatentClassification.生化学_微生物学: "Biochemistry; Beer; Spirits; Wine; Vinegar; Microbiology; Enzymology; Mutation or genetic engineering",
    PatentClassification.糖工業: "Sugar industry",
    PatentClassification.皮革: "Skins; Hides; Pelts; Leather",
    PatentClassification.鉄冶金: "Metallurgy of iron",
    PatentClassification.冶金_合金: "Metallurgy; Ferrous or non-ferrous alloys; Treatment of alloys or non-ferrous metals",
    PatentClassification.金属被覆_表面処理: "Coating metallic material",
    PatentClassification.電気分解_電気泳動: "Electrolytic or electrophoretic processes; Apparatus therefor",
    PatentClassification.結晶成長: "Crystal growth",
    PatentClassification.コンビナトリアル技術: "Combinatorial technology",
    PatentClassification.セクションC_その他: "Subject matter not otherwise provided for in this section",
    PatentClassification.糸_繊維_紡績: "Natural or man-made threads or fibres; Spinning",
    PatentClassification.糸_ロープ仕上げ: "Yarns; Mechanical finishing of yarns or ropes; Warping or beaming",
    PatentClassification.織成: "Weaving",
    PatentClassification.組みひも_メリヤス編成: "Braiding; Lace-making; Knitting; Trimmings; Non-woven fabrics",
    PatentClassification.縫製_刺しゅう: "Sewing; Embroidering; Tufting",
    PatentClassification.繊維処理_洗濯: "Treatment of textiles or the like; Laundering; Flexible materials not otherwise provided for",
    PatentClassification.ロープ_ケーブル: "Ropes; Cables other than electric",
    PatentClassification.製紙_セルロース: "Paper-making; Production of cellulose",
    PatentClassification.セクションD_その他: "Subject matter not otherwise provided for in this section",
    PatentClassification.道路_鉄道_橋りょう建設: "Construction of roads, railways, or bridges",
    PatentClassification.水工_基礎_土砂移送: "Hydraulic engineering; Foundations; Soil-shifting",
    PatentClassification.上水_下水: "Water supply; Sewerage",
    PatentClassification.建築物: "Building",
    PatentClassification.錠_鍵_金庫: "Locks; Keys; Window or door fittings; Safes",
    PatentClassification.戸_窓_シャッタ: "Doors, windows, shutters, or roller blinds in general; Ladders",
    PatentClassification.削孔_採鉱: "Earth or rock drilling; Mining",
    PatentClassification.セクションE_その他: "Subject matter not otherwise provided for in this section",
    PatentClassification.機関一般_蒸気機関: "Machines or engines in general; Engine plants in general; Steam engines",
    PatentClassification.燃焼機関: "Combustion engines; Hot-gas or combustion-product engine plants",
    PatentClassification.液体用機関_風力原動機: "Machines or engines for liquids; Wind, spring, or weight motors; Producing mechanical power or a reactive propulsive thrust",
    PatentClassification.容積形機械_ポンプ: "Positive-displacement machines for liquids; Pumps for liquids or elastic fluids",
    PatentClassification.流体圧アクチュエータ: "Fluid-pressure actuators; Hydraulics or pneumatics in general",
    PatentClassification.機械要素: "Engineering elements or units; General measures for producing and maintaining effective functioning of machines or installations",
    PatentClassification.ガス_液体の貯蔵分配: "Storing or distributing gases or liquids",
    PatentClassification.照明: "Lighting",
    PatentClassification.蒸気発生: "Steam generation",
    PatentClassification.燃焼装置: "Combustion apparatus; Combustion processes",
    PatentClassification.加熱_レンジ_換気: "Heating; Ranges; Ventilating",
    PatentClassification.冷凍_冷却_ヒートポンプ: "Refrigeration or cooling; Combined heating and refrigeration systems; Heat pump systems; Manufacture or storage of ice; Liquefaction or solidification of gases",
    PatentClassification.乾燥: "Drying",
    PatentClassification.炉_キルン_レトルト: "Furnaces; Kilns; Ovens; Retorts",
    PatentClassification.熱交換: "Heat exchange in general",
    PatentClassification.武器: "Weapons",
    PatentClassification.弾薬_爆破: "Ammunition; Blasting",
    PatentClassification.セクションF_その他: "Subject matter not otherwise provided for in this section",
    PatentClassification.測定_試験: "Measuring; Testing",
    PatentClassification.光学: "Optics",
    PatentClassification.写真_映画_ホログラフイ: "Photography; Cinematography; Analogous techniques using waves other than optical waves; Electrography; Holography",
    PatentClassification.時計: "Horology",
    PatentClassification.制御_調整: "Controlling; Regulating",
    PatentClassification.計算_計数: "Computing; Calculating; Counting",
    PatentClassification.チェック装置: "Checking-devices",
    PatentClassification.信号: "Signalling",
    PatentClassification.教育_暗号方法_表示_広告: "Educating; Cryptography; Display; Advertising; Seals",
    PatentClassification.楽器_音響: "Musical instruments; Acoustics",
    PatentClassification.情報記憶: "Information storage",
    PatentClassification.器械の細部: "Instrument details",
    PatentClassification.情報通信技術_ICT: "Information and communication technology specially adapted for specific application fields",
    PatentClassification.核物理_核工学: "Nuclear physics; Nuclear engineering",
    PatentClassification.セクションG_その他: "Subject matter not otherwise provided for in this section",
    PatentClassification.基本的電気素子: "Electric elements",
    PatentClassification.電力の発電_変換_配電: "Generation, conversion, or distribution of electric power",
    PatentClassification.基本電子回路: "Electronic circuitry",
    PatentClassification.電気通信技術: "Electric communication technique",
    PatentClassification.電気技術: "Electric techniques not otherwise provided for",
    PatentClassification.セクションH_その他: "Subject matter not otherwise provided for in this section",
}




# PatentClassification 英語 alias
AGRICULTURE = PatentClassification.農業_林業_畜産
BAKING = PatentClassification.ベイキング_生地製造
BUTCHERING = PatentClassification.屠殺_肉処理
FOODS = PatentClassification.食品_食料品
TOBACCO_PRODUCTS = PatentClassification.たばこ_喫煙具
WEARING_APPAREL = PatentClassification.衣類
HEADWEAR = PatentClassification.頭部着用物
FOOTWEAR = PatentClassification.履物
HABERDASHERY_JEWELLERY = PatentClassification.小間物_貴金属宝石
HAND_TRAVELLING_ARTICLES = PatentClassification.手持品_旅行用品
BRUSHWARE = PatentClassification.ブラシ製品
FURNITURE_DOMESTIC = PatentClassification.家具_家庭用品
MEDICAL_VETERINARY = PatentClassification.医学_獣医学_衛生学
LIFE_SAVING_FIRE_FIGHTING = PatentClassification.人命救助_消防
SPORTS_GAMES = PatentClassification.スポーツ_ゲーム_娯楽
SECTION_A_OTHER = PatentClassification.セクションA_その他
PHYSICAL_CHEMICAL_PROCESSES = PatentClassification.物理的化学的方法_装置
CRUSHING_PULVERISING = PatentClassification.破砕_粉砕
SEPARATION_BY_LIQUIDS = PatentClassification.固体物質の分離
CENTRIFUGAL_APPARATUS = PatentClassification.遠心装置
SPRAYING_ATOMISING = PatentClassification.霧化_噴霧
MECHANICAL_VIBRATIONS = PatentClassification.機械的振動
SEPARATING_SOLIDS_SORTING = PatentClassification.固体分離_仕分け
CLEANING = PatentClassification.清掃
SOLID_WASTE_DISPOSAL = PatentClassification.固体廃棄物処理
MECHANICAL_METAL_WORKING = PatentClassification.機械的金属加工
CASTING_POWDER_METALLURGY = PatentClassification.鋳造_粉末冶金
MACHINE_TOOLS = PatentClassification.工作機械_金属加工
GRINDING_POLISHING = PatentClassification.研削_研磨
HAND_TOOLS = PatentClassification.手工具_動力工具
CUTTING_TOOLS = PatentClassification.切断手工具_切断機
WOOD_WORKING = PatentClassification.木材加工
CEMENT_CLAY_STONE = PatentClassification.セメント_粘土_石材加工
PLASTICS_WORKING = PatentClassification.プラスチック加工
PRESSES = PatentClassification.プレス
PAPER_PRODUCTS_MANUFACTURING = PatentClassification.紙_板紙製品の製造
LAYERED_PRODUCTS = PatentClassification.積層体
ADDITIVE_MANUFACTURING = PatentClassification.付加製造技術
PRINTING = PatentClassification.印刷_タイプライター
BOOKBINDING = PatentClassification.製本_アルバム
WRITING_DRAWING_IMPLEMENTS = PatentClassification.筆記_製図用器具
DECORATIVE_ARTS = PatentClassification.装飾技術
VEHICLES_GENERAL = PatentClassification.車両一般
RAILWAYS = PatentClassification.鉄道
LAND_VEHICLES = PatentClassification.路面車両
SHIPS = PatentClassification.船舶_水上構造物
AIRCRAFT_AVIATION = PatentClassification.航空機_宇宙工学
CONVEYING_PACKING_STORING = PatentClassification.運搬_包装_貯蔵
HOISTING_LIFTING = PatentClassification.巻上装置_揚重装置
BOTTLE_OPENING_LIQUID_HANDLING = PatentClassification.容器開封密封_液体取扱い
SADDLERY_UPHOLSTERY = PatentClassification.馬具_詰め物
MICROSTRUCTURAL_TECHNOLOGY = PatentClassification.マイクロ構造技術
NANOTECHNOLOGY = PatentClassification.ナノテクノロジー
SECTION_B_OTHER = PatentClassification.セクションB_その他
INORGANIC_CHEMISTRY = PatentClassification.無機化学
WATER_TREATMENT = PatentClassification.水_廃水処理
GLASS_MINERAL_WOOL = PatentClassification.ガラス_鉱物
CEMENTS_CERAMICS = PatentClassification.セメント_コンクリート_セラミックス
FERTILISERS = PatentClassification.肥料
EXPLOSIVES_MATCHES = PatentClassification.火薬_マッチ
ORGANIC_CHEMISTRY = PatentClassification.有機化学
ORGANIC_MACROMOLECULAR = PatentClassification.有機高分子化合物
DYES_PAINTS_ADHESIVES = PatentClassification.染料_ペイント_接着剤
PETROLEUM_GAS_COKE = PatentClassification.石油_ガス_コークス工業
ANIMAL_VEGETABLE_OILS = PatentClassification.油脂_洗浄剤
BIOCHEMISTRY_MICROBIOLOGY = PatentClassification.生化学_微生物学
SUGAR_INDUSTRY = PatentClassification.糖工業
SKINS_LEATHER = PatentClassification.皮革
IRON_METALLURGY = PatentClassification.鉄冶金
METALLURGY_ALLOYS = PatentClassification.冶金_合金
METALLIC_COATING = PatentClassification.金属被覆_表面処理
ELECTROLYSIS = PatentClassification.電気分解_電気泳動
CRYSTAL_GROWTH = PatentClassification.結晶成長
COMBINATORIAL_TECHNOLOGY = PatentClassification.コンビナトリアル技術
SECTION_C_OTHER = PatentClassification.セクションC_その他
THREADS_FIBRES_SPINNING = PatentClassification.糸_繊維_紡績
YARNS_ROPES = PatentClassification.糸_ロープ仕上げ
WEAVING = PatentClassification.織成
BRAIDING_KNITTING = PatentClassification.組みひも_メリヤス編成
SEWING_EMBROIDERING = PatentClassification.縫製_刺しゅう
TEXTILE_TREATMENT = PatentClassification.繊維処理_洗濯
ROPES_CABLES = PatentClassification.ロープ_ケーブル
PAPER_MAKING = PatentClassification.製紙_セルロース
SECTION_D_OTHER = PatentClassification.セクションD_その他
ROAD_RAILWAY_BRIDGE_CONSTRUCTION = PatentClassification.道路_鉄道_橋りょう建設
HYDRAULIC_ENGINEERING = PatentClassification.水工_基礎_土砂移送
WATER_SUPPLY_SEWERAGE = PatentClassification.上水_下水
BUILDING = PatentClassification.建築物
LOCKS_KEYS_SAFES = PatentClassification.錠_鍵_金庫
DOORS_WINDOWS = PatentClassification.戸_窓_シャッタ
EARTH_DRILLING_MINING = PatentClassification.削孔_採鉱
SECTION_E_OTHER = PatentClassification.セクションE_その他
ENGINES_GENERAL = PatentClassification.機関一般_蒸気機関
COMBUSTION_ENGINES = PatentClassification.燃焼機関
LIQUID_MACHINES_WIND_MOTORS = PatentClassification.液体用機関_風力原動機
PUMPS = PatentClassification.容積形機械_ポンプ
FLUID_PRESSURE_ACTUATORS = PatentClassification.流体圧アクチュエータ
ENGINEERING_ELEMENTS = PatentClassification.機械要素
GAS_LIQUID_STORAGE = PatentClassification.ガス_液体の貯蔵分配
LIGHTING = PatentClassification.照明
STEAM_GENERATION = PatentClassification.蒸気発生
COMBUSTION_APPARATUS = PatentClassification.燃焼装置
HEATING_VENTILATING = PatentClassification.加熱_レンジ_換気
REFRIGERATION_COOLING = PatentClassification.冷凍_冷却_ヒートポンプ
DRYING = PatentClassification.乾燥
FURNACES_KILNS = PatentClassification.炉_キルン_レトルト
HEAT_EXCHANGE = PatentClassification.熱交換
WEAPONS = PatentClassification.武器
AMMUNITION_BLASTING = PatentClassification.弾薬_爆破
SECTION_F_OTHER = PatentClassification.セクションF_その他
MEASURING_TESTING = PatentClassification.測定_試験
OPTICS = PatentClassification.光学
PHOTOGRAPHY_CINEMATOGRAPHY = PatentClassification.写真_映画_ホログラフイ
HOROLOGY = PatentClassification.時計
CONTROLLING_REGULATING = PatentClassification.制御_調整
COMPUTING_CALCULATING = PatentClassification.計算_計数
CHECKING_DEVICES = PatentClassification.チェック装置
SIGNALLING = PatentClassification.信号
EDUCATION_CRYPTOGRAPHY_DISPLAY = PatentClassification.教育_暗号方法_表示_広告
MUSICAL_INSTRUMENTS_ACOUSTICS = PatentClassification.楽器_音響
INFORMATION_STORAGE = PatentClassification.情報記憶
INSTRUMENT_DETAILS = PatentClassification.器械の細部
ICT_SPECIFIC_APPLICATIONS = PatentClassification.情報通信技術_ICT
NUCLEAR_PHYSICS = PatentClassification.核物理_核工学
SECTION_G_OTHER = PatentClassification.セクションG_その他
ELECTRIC_ELEMENTS = PatentClassification.基本的電気素子
ELECTRIC_POWER = PatentClassification.電力の発電_変換_配電
ELECTRONIC_CIRCUITRY = PatentClassification.基本電子回路
ELECTRIC_COMMUNICATION = PatentClassification.電気通信技術
ELECTRIC_TECHNIQUES = PatentClassification.電気技術
SECTION_H_OTHER = PatentClassification.セクションH_その他


class DesignClassification(StrEnum):
    """意匠の日本意匠分類の列挙型。

    日本意匠分類に基づく分類コードを定義する。
    ``description_ja`` プロパティで日本語の分類名を取得できる。
    """

    製造食品及び嗜好品_その他 = "A0"
    製造食品及び嗜好品 = "A1"
    衣服及び身の回り品_その他 = "B0"
    衣服 = "B1"
    服飾品 = "B2"
    身の回り品 = "B3"
    かばん_携帯用袋物 = "B4"
    履物 = "B5"
    喫煙用具_点火器 = "B6"
    化粧用具_理容用具 = "B7"
    衣服及び身の回り品_汎用部品 = "B9"
    生活用品_その他 = "C0"
    寝具_床敷物_カーテン = "C1"
    室内装飾品 = "C2"
    清掃用具_洗濯用具 = "C3"
    家庭用保健衛生用品 = "C4"
    飲食用容器_調理用容器 = "C5"
    飲食用具_調理用器具 = "C6"
    慶弔用品 = "C7"
    住宅設備用品_その他 = "D0"
    発光具_照明器具 = "D3"
    暖冷房機_空調換気機器 = "D4"
    厨房設備用品_衛生設備用品 = "D5"
    室内整理用家具_用具 = "D6"
    家具 = "D7"
    住宅設備用品_汎用部品 = "D9"
    趣味娯楽_運動競技用品_その他 = "E0"
    おもちゃ = "E1"
    遊戯娯楽用品 = "E2"
    運動競技用品 = "E3"
    楽器 = "E4"
    事務用品_販売用品_その他 = "F0"
    教習具_書画用品 = "F1"
    筆記具_事務用具 = "F2"
    事務用紙製品_印刷物 = "F3"
    包装紙_包装用容器 = "F4"
    広告用具_表示具_商品陳列用具 = "F5"
    運輸_運搬機械_その他 = "G0"
    運搬_昇降_貨物取扱い機械器具 = "G1"
    車両 = "G2"
    船舶 = "G3"
    航空機 = "G4"
    電気電子_通信機械器具_その他 = "H0"
    基本的電気素子 = "H1"
    回転電気機械_配電機械器具 = "H2"
    電子情報処理_記憶機械器具 = "H6"
    電子情報入出力機器 = "H7"
    一般機械器具_その他 = "J0"
    計量器_測定機械器具 = "J1"
    時計 = "J2"
    光学機械器具 = "J3"
    事務用機械器具 = "J4"
    自動販売機_自動サービス機 = "J5"
    保安機械器具 = "J6"
    医療機械器具 = "J7"
    産業用機械器具_その他 = "K0"
    利器_工具 = "K1"
    漁業用機械器具 = "K2"
    農業用機械器具_建設機械 = "K3"
    食料加工機械 = "K4"
    繊維機械_ミシン = "K5"
    化学機械器具 = "K6"
    金属加工機械_木材加工機械 = "K7"
    動力機械器具_ポンプ = "K8"
    産業用機械器具_汎用部品 = "K9"
    土木建築用品_その他 = "L0"
    仮設工事用品 = "L1"
    土木構造物_土木用品 = "L2"
    組立て家屋_屋外装備品 = "L3"
    建築用構成品 = "L4"
    建築用内外装材 = "L6"
    建物用構造材_枠材 = "L7"
    基礎製品_その他 = "M0"
    織物地_板_ひも = "M1"
    配線配管用管_バルブ = "M2"
    ねじ_くぎ_金物 = "M3"
    物品_その他 = "N0"
    ソフトウェア = "N1"
    ウェブサイト = "N2"
    画像_その他 = "W10"
    情報入力操作用画像 = "W11"
    機能実行操作用画像 = "W12"
    情報閲覧表示用画像 = "W13"
    複合画像 = "W14"
    画像構成部品 = "W19"

    @property
    def description_ja(self) -> str:
        """日本語の分類名を返す。

        Returns:
            日本語の分類名文字列。
        """
        return _DESIGN_DESC_JA[self]


_DESIGN_DESC_JA: dict[DesignClassification, str] = {
    DesignClassification.製造食品及び嗜好品_その他: "A1に属さないその他の製造食品及び嗜好品",
    DesignClassification.製造食品及び嗜好品: "製造食品及び嗜好品",
    DesignClassification.衣服及び身の回り品_その他: "B1～B9に属さないその他の衣服及び身の回り品",
    DesignClassification.衣服: "衣服",
    DesignClassification.服飾品: "服飾品",
    DesignClassification.身の回り品: "身の回り品",
    DesignClassification.かばん_携帯用袋物: "かばん又は携帯用袋物",
    DesignClassification.履物: "履物",
    DesignClassification.喫煙用具_点火器: "喫煙用具及び点火器",
    DesignClassification.化粧用具_理容用具: "化粧用具又は理容用具",
    DesignClassification.衣服及び身の回り品_汎用部品: "衣服及び身の回り品汎用部品及び付属品",
    DesignClassification.生活用品_その他: "C1～C7に属さないその他の生活用品",
    DesignClassification.寝具_床敷物_カーテン: "寝具、床敷物、カーテン等",
    DesignClassification.室内装飾品: "室内装飾品",
    DesignClassification.清掃用具_洗濯用具: "清掃用具、洗濯用具等",
    DesignClassification.家庭用保健衛生用品: "家庭用保健衛生用品",
    DesignClassification.飲食用容器_調理用容器: "飲食用容器又は調理用容器",
    DesignClassification.飲食用具_調理用器具: "飲食用具及び調理用器具",
    DesignClassification.慶弔用品: "慶弔用品",
    DesignClassification.住宅設備用品_その他: "D3～D9に属さないその他の住宅設備用品",
    DesignClassification.発光具_照明器具: "発光具及び照明器具",
    DesignClassification.暖冷房機_空調換気機器: "暖冷房機又は空調換気機器",
    DesignClassification.厨房設備用品_衛生設備用品: "厨房設備用品及び衛生設備用品",
    DesignClassification.室内整理用家具_用具: "室内整理用家具・用具",
    DesignClassification.家具: "家具",
    DesignClassification.住宅設備用品_汎用部品: "住宅設備用品汎用部品及び付属品",
    DesignClassification.趣味娯楽_運動競技用品_その他: "E1～E4に属さないその他の趣味娯楽用品及び運動競技用品",
    DesignClassification.おもちゃ: "おもちゃ",
    DesignClassification.遊戯娯楽用品: "遊戯娯楽用品",
    DesignClassification.運動競技用品: "運動競技用品",
    DesignClassification.楽器: "楽器",
    DesignClassification.事務用品_販売用品_その他: "F1～F5に属さないその他の事務用品及び販売用品",
    DesignClassification.教習具_書画用品: "教習具、書画用品等",
    DesignClassification.筆記具_事務用具: "筆記具、事務用具等",
    DesignClassification.事務用紙製品_印刷物: "事務用紙製品、印刷物等",
    DesignClassification.包装紙_包装用容器: "包装紙、包装用容器等",
    DesignClassification.広告用具_表示具_商品陳列用具: "広告用具、表示具及び商品陳列用具",
    DesignClassification.運輸_運搬機械_その他: "G1～G4に属さないその他の運輸又は運搬機械",
    DesignClassification.運搬_昇降_貨物取扱い機械器具: "運搬、昇降又は貨物取扱い用機械器具",
    DesignClassification.車両: "車両",
    DesignClassification.船舶: "船舶",
    DesignClassification.航空機: "航空機",
    DesignClassification.電気電子_通信機械器具_その他: "H1～H7に属さないその他の電気・電子機械器具及び通信機械器具",
    DesignClassification.基本的電気素子: "基本的電気素子",
    DesignClassification.回転電気機械_配電機械器具: "回転電気機械、配電機械器具",
    DesignClassification.電子情報処理_記憶機械器具: "電子情報処理・記憶機械器具",
    DesignClassification.電子情報入出力機器: "電子情報入出力機器",
    DesignClassification.一般機械器具_その他: "J1～J7に属さないその他の一般機械器具",
    DesignClassification.計量器_測定機械器具: "計量器、測定機械器具及び測量機械器具",
    DesignClassification.時計: "時計",
    DesignClassification.光学機械器具: "光学機械器具",
    DesignClassification.事務用機械器具: "事務用機械器具",
    DesignClassification.自動販売機_自動サービス機: "自動販売機及び自動サービス機",
    DesignClassification.保安機械器具: "保安機械器具等",
    DesignClassification.医療機械器具: "医療機械器具",
    DesignClassification.産業用機械器具_その他: "K1～K9に属さないその他の産業用機械器具",
    DesignClassification.利器_工具: "利器及び工具",
    DesignClassification.漁業用機械器具: "漁業用機械器具",
    DesignClassification.農業用機械器具_建設機械: "農業用機械器具、鉱山機械、建設機械等",
    DesignClassification.食料加工機械: "食料加工機械等",
    DesignClassification.繊維機械_ミシン: "繊維機械及びミシン",
    DesignClassification.化学機械器具: "化学機械器具",
    DesignClassification.金属加工機械_木材加工機械: "金属加工機械、木材加工機械等",
    DesignClassification.動力機械器具_ポンプ: "動力機械器具、ポンプ、圧縮機、送風機等",
    DesignClassification.産業用機械器具_汎用部品: "産業用機械器具汎用部品及び付属品",
    DesignClassification.土木建築用品_その他: "L1～L7に属さないその他の土木建築用品",
    DesignClassification.仮設工事用品: "仮設工事用品",
    DesignClassification.土木構造物_土木用品: "土木構造物及び土木用品",
    DesignClassification.組立て家屋_屋外装備品: "組立て家屋、屋外装備品等",
    DesignClassification.建築用構成品: "建築用構成品",
    DesignClassification.建築用内外装材: "建築用内外装材",
    DesignClassification.建物用構造材_枠材: "建物用構造材、枠材等",
    DesignClassification.基礎製品_その他: "M1～M3に属さないその他の基礎製品",
    DesignClassification.織物地_板_ひも: "織物地、板、ひも等",
    DesignClassification.配線配管用管_バルブ: "配線・配管用管、管継ぎ手、バルブ等",
    DesignClassification.ねじ_くぎ_金物: "ねじ、くぎ、開閉金物、係止具等",
    DesignClassification.物品_その他: "N1、N2に属さないその他の物品等",
    DesignClassification.ソフトウェア: "ソフトウェア",
    DesignClassification.ウェブサイト: "ウェブサイト",
    DesignClassification.画像_その他: "W11～W19に属さないその他の画像",
    DesignClassification.情報入力操作用画像: "情報入力操作用画像",
    DesignClassification.機能実行操作用画像: "機能実行操作用画像",
    DesignClassification.情報閲覧表示用画像: "情報閲覧表示用画像",
    DesignClassification.複合画像: "複合画像",
    DesignClassification.画像構成部品: "画像構成部品",
}




# DesignClassification 英語 alias
DESIGN_FOOD_OTHER = DesignClassification.製造食品及び嗜好品_その他
DESIGN_FOOD = DesignClassification.製造食品及び嗜好品
DESIGN_CLOTHING_ACCESSORIES_OTHER = DesignClassification.衣服及び身の回り品_その他
DESIGN_CLOTHING = DesignClassification.衣服
DESIGN_FASHION = DesignClassification.服飾品
DESIGN_PERSONAL_ITEMS = DesignClassification.身の回り品
DESIGN_BAGS = DesignClassification.かばん_携帯用袋物
DESIGN_FOOTWEAR = DesignClassification.履物
DESIGN_SMOKING = DesignClassification.喫煙用具_点火器
DESIGN_COSMETICS = DesignClassification.化粧用具_理容用具
DESIGN_CLOTHING_PARTS = DesignClassification.衣服及び身の回り品_汎用部品
DESIGN_HOUSEHOLD_OTHER = DesignClassification.生活用品_その他
DESIGN_BEDDING = DesignClassification.寝具_床敷物_カーテン
DESIGN_INTERIOR_DECOR = DesignClassification.室内装飾品
DESIGN_CLEANING = DesignClassification.清掃用具_洗濯用具
DESIGN_HEALTH_HYGIENE = DesignClassification.家庭用保健衛生用品
DESIGN_FOOD_CONTAINERS = DesignClassification.飲食用容器_調理用容器
DESIGN_TABLEWARE = DesignClassification.飲食用具_調理用器具
DESIGN_CEREMONIAL = DesignClassification.慶弔用品
DESIGN_HOME_EQUIPMENT_OTHER = DesignClassification.住宅設備用品_その他
DESIGN_LIGHTING = DesignClassification.発光具_照明器具
DESIGN_HVAC = DesignClassification.暖冷房機_空調換気機器
DESIGN_KITCHEN_SANITARY = DesignClassification.厨房設備用品_衛生設備用品
DESIGN_STORAGE_FURNITURE = DesignClassification.室内整理用家具_用具
DESIGN_FURNITURE = DesignClassification.家具
DESIGN_HOME_EQUIPMENT_PARTS = DesignClassification.住宅設備用品_汎用部品
DESIGN_HOBBY_SPORTS_OTHER = DesignClassification.趣味娯楽_運動競技用品_その他
DESIGN_TOYS = DesignClassification.おもちゃ
DESIGN_GAMES = DesignClassification.遊戯娯楽用品
DESIGN_SPORTS = DesignClassification.運動競技用品
DESIGN_MUSICAL_INSTRUMENTS = DesignClassification.楽器
DESIGN_OFFICE_SALES_OTHER = DesignClassification.事務用品_販売用品_その他
DESIGN_EDUCATIONAL_ART = DesignClassification.教習具_書画用品
DESIGN_WRITING_OFFICE = DesignClassification.筆記具_事務用具
DESIGN_PAPER_PRODUCTS = DesignClassification.事務用紙製品_印刷物
DESIGN_PACKAGING = DesignClassification.包装紙_包装用容器
DESIGN_ADVERTISING_DISPLAY = DesignClassification.広告用具_表示具_商品陳列用具
DESIGN_TRANSPORT_OTHER = DesignClassification.運輸_運搬機械_その他
DESIGN_MATERIAL_HANDLING = DesignClassification.運搬_昇降_貨物取扱い機械器具
DESIGN_VEHICLES = DesignClassification.車両
DESIGN_SHIPS = DesignClassification.船舶
DESIGN_AIRCRAFT = DesignClassification.航空機
DESIGN_ELECTRONICS_OTHER = DesignClassification.電気電子_通信機械器具_その他
DESIGN_ELECTRIC_ELEMENTS = DesignClassification.基本的電気素子
DESIGN_ELECTRICAL_MACHINES = DesignClassification.回転電気機械_配電機械器具
DESIGN_COMPUTERS = DesignClassification.電子情報処理_記憶機械器具
DESIGN_IO_DEVICES = DesignClassification.電子情報入出力機器
DESIGN_GENERAL_MACHINERY_OTHER = DesignClassification.一般機械器具_その他
DESIGN_MEASURING = DesignClassification.計量器_測定機械器具
DESIGN_CLOCKS = DesignClassification.時計
DESIGN_OPTICAL = DesignClassification.光学機械器具
DESIGN_OFFICE_MACHINES = DesignClassification.事務用機械器具
DESIGN_VENDING_MACHINES = DesignClassification.自動販売機_自動サービス機
DESIGN_SECURITY = DesignClassification.保安機械器具
DESIGN_MEDICAL = DesignClassification.医療機械器具
DESIGN_INDUSTRIAL_OTHER = DesignClassification.産業用機械器具_その他
DESIGN_TOOLS = DesignClassification.利器_工具
DESIGN_FISHING = DesignClassification.漁業用機械器具
DESIGN_AGRICULTURAL_CONSTRUCTION = DesignClassification.農業用機械器具_建設機械
DESIGN_FOOD_PROCESSING = DesignClassification.食料加工機械
DESIGN_TEXTILE_MACHINES = DesignClassification.繊維機械_ミシン
DESIGN_CHEMICAL_EQUIPMENT = DesignClassification.化学機械器具
DESIGN_METALWORKING = DesignClassification.金属加工機械_木材加工機械
DESIGN_POWER_MACHINES = DesignClassification.動力機械器具_ポンプ
DESIGN_INDUSTRIAL_PARTS = DesignClassification.産業用機械器具_汎用部品
DESIGN_CIVIL_BUILDING_OTHER = DesignClassification.土木建築用品_その他
DESIGN_TEMPORARY_CONSTRUCTION = DesignClassification.仮設工事用品
DESIGN_CIVIL_STRUCTURES = DesignClassification.土木構造物_土木用品
DESIGN_PREFAB_OUTDOOR = DesignClassification.組立て家屋_屋外装備品
DESIGN_BUILDING_COMPONENTS = DesignClassification.建築用構成品
DESIGN_BUILDING_MATERIALS = DesignClassification.建築用内外装材
DESIGN_STRUCTURAL_MATERIALS = DesignClassification.建物用構造材_枠材
DESIGN_BASIC_PRODUCTS_OTHER = DesignClassification.基礎製品_その他
DESIGN_TEXTILES_BOARDS = DesignClassification.織物地_板_ひも
DESIGN_PIPES_VALVES = DesignClassification.配線配管用管_バルブ
DESIGN_FASTENERS = DesignClassification.ねじ_くぎ_金物
DESIGN_OTHER_GOODS = DesignClassification.物品_その他
DESIGN_SOFTWARE = DesignClassification.ソフトウェア
DESIGN_WEBSITE = DesignClassification.ウェブサイト
DESIGN_IMAGES_OTHER = DesignClassification.画像_その他
DESIGN_INPUT_IMAGES = DesignClassification.情報入力操作用画像
DESIGN_FUNCTION_IMAGES = DesignClassification.機能実行操作用画像
DESIGN_DISPLAY_IMAGES = DesignClassification.情報閲覧表示用画像
DESIGN_COMPOSITE_IMAGES = DesignClassification.複合画像
DESIGN_IMAGE_COMPONENTS = DesignClassification.画像構成部品


class TrademarkClassification(StrEnum):
    """商標の類似商品・役務審査基準に基づく区分の列挙型。

    ニース国際分類に対応する第1類～第45類の区分コードを定義する。
    第1類～第34類は商品区分、第35類～第45類は役務区分に相当する。
    ``description_ja`` プロパティで日本語の区分名を取得できる。
    """

    化学品 = "1"
    塗料_着色料 = "2"
    洗浄剤_化粧品 = "3"
    工業用油_燃料 = "4"
    薬剤 = "5"
    卑金属_製品 = "6"
    加工機械_原動機 = "7"
    手動工具 = "8"
    科学用機械器具 = "9"
    医療用機械器具 = "10"
    照明_加熱_冷却装置 = "11"
    乗物_移動装置 = "12"
    火器_火工品 = "13"
    貴金属_宝飾品_時計 = "14"
    楽器 = "15"
    紙_事務用品 = "16"
    絶縁_断熱材料 = "17"
    革_旅行用品_馬具 = "18"
    建築材料 = "19"
    家具_プラスチック製品 = "20"
    家庭用器具_ガラス_磁器 = "21"
    ロープ_帆布_繊維 = "22"
    織物用の糸 = "23"
    織物_織物製カバー = "24"
    被服_履物 = "25"
    裁縫用品 = "26"
    床敷物_壁掛け = "27"
    がん具_遊戯用具_運動用具 = "28"
    動物性食品_加工野菜 = "29"
    植物性食品_調味料 = "30"
    陸産物_動植物_飼料 = "31"
    ノンアルコール飲料_ビール = "32"
    アルコール飲料 = "33"
    たばこ_喫煙用具 = "34"
    広告_事業管理 = "35"
    金融_保険_不動産 = "36"
    建設_修理 = "37"
    電気通信 = "38"
    輸送_保管_旅行手配 = "39"
    物品の加工処理 = "40"
    教育_娯楽_スポーツ = "41"
    科学技術_ソフトウェア開発 = "42"
    飲食物_宿泊施設 = "43"
    医療_美容_農林業 = "44"
    冠婚葬祭_警備_法律 = "45"

    @property
    def description_ja(self) -> str:
        """日本語の区分名を返す。

        Returns:
            日本語の区分名文字列。
        """
        return _TRADEMARK_DESC_JA[self]


_TRADEMARK_DESC_JA: dict[TrademarkClassification, str] = {
    TrademarkClassification.化学品: "工業用、科学用又は農業用の化学品",
    TrademarkClassification.塗料_着色料: "塗料、着色料及び腐食の防止用の調製品",
    TrademarkClassification.洗浄剤_化粧品: "洗浄剤及び化粧品",
    TrademarkClassification.工業用油_燃料: "工業用油、工業用油脂、燃料及び光剤",
    TrademarkClassification.薬剤: "薬剤",
    TrademarkClassification.卑金属_製品: "卑金属及びその製品",
    TrademarkClassification.加工機械_原動機: "加工機械、原動機(陸上の乗物用のものを除く。)その他の機械",
    TrademarkClassification.手動工具: "手動工具",
    TrademarkClassification.科学用機械器具: "科学用、航海用、測量用、写真用、音響用、映像用、計量用、信号用、検査用、救命用、教育用、計算用又は情報処理用の機械器具、光学式の機械器具及び電気の伝導用、電気回路の開閉用、変圧用、蓄電用、電圧調整用又は電気制御用の機械器具",
    TrademarkClassification.医療用機械器具: "医療用機械器具及び医療用品",
    TrademarkClassification.照明_加熱_冷却装置: "照明用、加熱用、蒸気発生用、調理用、冷却用、乾燥用、換気用、給水用又は衛生用の装置",
    TrademarkClassification.乗物_移動装置: "乗物その他移動用の装置",
    TrademarkClassification.火器_火工品: "火器及び火工品",
    TrademarkClassification.貴金属_宝飾品_時計: "貴金属、貴金属製品であって他の類に属しないもの、宝飾品及び時計",
    TrademarkClassification.楽器: "楽器",
    TrademarkClassification.紙_事務用品: "紙、紙製品及び事務用品",
    TrademarkClassification.絶縁_断熱材料: "電気絶縁用、断熱用又は防音用の材料及び材料用のプラスチック",
    TrademarkClassification.革_旅行用品_馬具: "革及びその模造品、旅行用品並びに馬具",
    TrademarkClassification.建築材料: "金属製でない建築材料",
    TrademarkClassification.家具_プラスチック製品: "家具及びプラスチック製品であって他の類に属しないもの",
    TrademarkClassification.家庭用器具_ガラス_磁器: "家庭用又は台所用の手動式の器具、化粧用具、ガラス製品及び磁器製品",
    TrademarkClassification.ロープ_帆布_繊維: "ロープ製品、帆布製品、詰物用の材料及び織物用の原料繊維",
    TrademarkClassification.織物用の糸: "織物用の糸",
    TrademarkClassification.織物_織物製カバー: "織物及び家庭用の織物製カバー",
    TrademarkClassification.被服_履物: "被服及び履物",
    TrademarkClassification.裁縫用品: "裁縫用品",
    TrademarkClassification.床敷物_壁掛け: "床敷物及び織物製でない壁掛け",
    TrademarkClassification.がん具_遊戯用具_運動用具: "がん具、遊戯用具及び運動用具",
    TrademarkClassification.動物性食品_加工野菜: "動物性の食品及び加工した野菜その他の食用園芸作物",
    TrademarkClassification.植物性食品_調味料: "加工した植物性の食品(他の類に属するものを除く。)及び調味料",
    TrademarkClassification.陸産物_動植物_飼料: "加工していない陸産物、生きている動植物及び飼料",
    TrademarkClassification.ノンアルコール飲料_ビール: "アルコールを含有しない飲料及びビール",
    TrademarkClassification.アルコール飲料: "ビールを除くアルコール飲料",
    TrademarkClassification.たばこ_喫煙用具: "たばこ、喫煙用具及びマッチ",
    TrademarkClassification.広告_事業管理: "広告、事業の管理又は運営及び事務処理及び小売又は卸売の業務において行われる顧客に対する便益の提供",
    TrademarkClassification.金融_保険_不動産: "金融、保険及び不動産の取引",
    TrademarkClassification.建設_修理: "建設、設置工事及び修理",
    TrademarkClassification.電気通信: "電気通信",
    TrademarkClassification.輸送_保管_旅行手配: "輸送、こん包及び保管並びに旅行の手配",
    TrademarkClassification.物品の加工処理: "物品の加工その他の処理",
    TrademarkClassification.教育_娯楽_スポーツ: "教育、訓練、娯楽、スポーツ及び文化活動",
    TrademarkClassification.科学技術_ソフトウェア開発: "科学技術又は産業に関する調査研究及び設計並びに電子計算機又はソフトウェアの設計及び開発",
    TrademarkClassification.飲食物_宿泊施設: "飲食物の提供及び宿泊施設の提供",
    TrademarkClassification.医療_美容_農林業: "医療、動物の治療、人又は動物に関する衛生及び美容並びに農業、園芸又は林業に係る役務",
    TrademarkClassification.冠婚葬祭_警備_法律: "冠婚葬祭に係る役務その他の個人の需要に応じて提供する役務(他の類に属するものを除く。)、警備及び法律事務",
}




# TrademarkClassification 英語 alias
TM_CHEMICALS = TrademarkClassification.化学品
TM_PAINTS_COATINGS = TrademarkClassification.塗料_着色料
TM_CLEANING_COSMETICS = TrademarkClassification.洗浄剤_化粧品
TM_OILS_FUELS = TrademarkClassification.工業用油_燃料
TM_PHARMACEUTICALS = TrademarkClassification.薬剤
TM_BASE_METALS = TrademarkClassification.卑金属_製品
TM_MACHINES_ENGINES = TrademarkClassification.加工機械_原動機
TM_HAND_TOOLS = TrademarkClassification.手動工具
TM_SCIENTIFIC_INSTRUMENTS = TrademarkClassification.科学用機械器具
TM_MEDICAL_INSTRUMENTS = TrademarkClassification.医療用機械器具
TM_LIGHTING_HEATING_COOLING = TrademarkClassification.照明_加熱_冷却装置
TM_VEHICLES = TrademarkClassification.乗物_移動装置
TM_FIREARMS = TrademarkClassification.火器_火工品
TM_PRECIOUS_METALS_JEWELLERY = TrademarkClassification.貴金属_宝飾品_時計
TM_MUSICAL_INSTRUMENTS = TrademarkClassification.楽器
TM_PAPER_OFFICE = TrademarkClassification.紙_事務用品
TM_INSULATING_MATERIALS = TrademarkClassification.絶縁_断熱材料
TM_LEATHER_TRAVEL = TrademarkClassification.革_旅行用品_馬具
TM_BUILDING_MATERIALS = TrademarkClassification.建築材料
TM_FURNITURE_PLASTICS = TrademarkClassification.家具_プラスチック製品
TM_HOUSEHOLD_UTENSILS = TrademarkClassification.家庭用器具_ガラス_磁器
TM_ROPES_TEXTILES = TrademarkClassification.ロープ_帆布_繊維
TM_YARNS = TrademarkClassification.織物用の糸
TM_TEXTILES_COVERS = TrademarkClassification.織物_織物製カバー
TM_CLOTHING_FOOTWEAR = TrademarkClassification.被服_履物
TM_SEWING = TrademarkClassification.裁縫用品
TM_FLOOR_WALL_COVERINGS = TrademarkClassification.床敷物_壁掛け
TM_TOYS_GAMES_SPORTS = TrademarkClassification.がん具_遊戯用具_運動用具
TM_ANIMAL_FOODS = TrademarkClassification.動物性食品_加工野菜
TM_PLANT_FOODS = TrademarkClassification.植物性食品_調味料
TM_RAW_AGRICULTURAL = TrademarkClassification.陸産物_動植物_飼料
TM_NON_ALCOHOLIC = TrademarkClassification.ノンアルコール飲料_ビール
TM_ALCOHOLIC = TrademarkClassification.アルコール飲料
TM_TOBACCO = TrademarkClassification.たばこ_喫煙用具
TM_ADVERTISING = TrademarkClassification.広告_事業管理
TM_FINANCE_INSURANCE = TrademarkClassification.金融_保険_不動産
TM_CONSTRUCTION_REPAIR = TrademarkClassification.建設_修理
TM_TELECOMMUNICATIONS = TrademarkClassification.電気通信
TM_TRANSPORT_STORAGE = TrademarkClassification.輸送_保管_旅行手配
TM_MATERIAL_TREATMENT = TrademarkClassification.物品の加工処理
TM_EDUCATION_ENTERTAINMENT = TrademarkClassification.教育_娯楽_スポーツ
TM_TECHNOLOGY_SOFTWARE = TrademarkClassification.科学技術_ソフトウェア開発
TM_FOOD_ACCOMMODATION = TrademarkClassification.飲食物_宿泊施設
TM_MEDICAL_BEAUTY = TrademarkClassification.医療_美容_農林業
TM_CEREMONIES_SECURITY_LAW = TrademarkClassification.冠婚葬祭_警備_法律



"""職場情報の区分コード列挙型モジュール。

従業員の平均年齢、平均継続勤務年数、月平均所定外労働時間、
女性労働者割合の区分コードを定義する。
"""

from __future__ import annotations

from enum import StrEnum


class AverageAge(StrEnum):
    """従業員の平均年齢の区分コード。

    検索時に average_age パラメータとして指定する。
    ``label`` / ``label_en`` プロパティで日英のラベルを取得できる。
    """

    歳30以下 = "A"
    歳31から45 = "B"
    歳46から60 = "C"
    歳61以上 = "D"

    @property
    def label(self) -> str:
        """日本語ラベルを返す。"""
        return _AVERAGE_AGE_LABELS[self][0]

    @property
    def label_en(self) -> str:
        """英語ラベルを返す。"""
        return _AVERAGE_AGE_LABELS[self][1]


_AVERAGE_AGE_LABELS: dict[AverageAge, tuple[str, str]] = {
    AverageAge.歳30以下: ("～30歳", "30 or under"),
    AverageAge.歳31から45: ("31歳～45歳", "31-45"),
    AverageAge.歳46から60: ("46歳～60歳", "46-60"),
    AverageAge.歳61以上: ("61歳～", "61 or over"),
}


class AverageContinuousServiceYears(StrEnum):
    """平均継続勤務年数の区分コード。

    検索時に average_continuous_service_years パラメータとして指定する。
    ``label`` / ``label_en`` プロパティで日英のラベルを取得できる。
    """

    年5以下 = "A"
    年6から10 = "B"
    年11から20 = "C"
    年21以上 = "D"

    @property
    def label(self) -> str:
        """日本語ラベルを返す。"""
        return _SERVICE_YEARS_LABELS[self][0]

    @property
    def label_en(self) -> str:
        """英語ラベルを返す。"""
        return _SERVICE_YEARS_LABELS[self][1]


_SERVICE_YEARS_LABELS: dict[AverageContinuousServiceYears, tuple[str, str]] = {
    AverageContinuousServiceYears.年5以下: ("～5年", "5 years or less"),
    AverageContinuousServiceYears.年6から10: ("6年～10年", "6-10 years"),
    AverageContinuousServiceYears.年11から20: ("11年～20年", "11-20 years"),
    AverageContinuousServiceYears.年21以上: ("21年～", "21 years or more"),
}


class MonthAverageOvertimeHours(StrEnum):
    """月平均所定外労働時間の区分コード。

    検索時に month_average_predetermined_overtime_hours パラメータとして指定する。
    ``label`` / ``label_en`` プロパティで日英のラベルを取得できる。
    """

    時間20未満 = "A"
    時間40未満 = "B"
    時間40以上 = "C"

    @property
    def label(self) -> str:
        """日本語ラベルを返す。"""
        return _OVERTIME_LABELS[self][0]

    @property
    def label_en(self) -> str:
        """英語ラベルを返す。"""
        return _OVERTIME_LABELS[self][1]


_OVERTIME_LABELS: dict[MonthAverageOvertimeHours, tuple[str, str]] = {
    MonthAverageOvertimeHours.時間20未満: ("20時間未満", "Less than 20 hours"),
    MonthAverageOvertimeHours.時間40未満: ("40時間未満", "Less than 40 hours"),
    MonthAverageOvertimeHours.時間40以上: ("40時間以上", "40 hours or more"),
}


class FemaleWorkersProportion(StrEnum):
    """労働者に占める女性労働者の割合の区分コード。

    検索時に female_workers_proportion パラメータとして指定する。
    ``label`` / ``label_en`` プロパティで日英のラベルを取得できる。
    """

    割合20以下 = "A"
    割合21から40 = "B"
    割合41から60 = "C"
    割合61以上 = "D"

    @property
    def label(self) -> str:
        """日本語ラベルを返す。"""
        return _FEMALE_LABELS[self][0]

    @property
    def label_en(self) -> str:
        """英語ラベルを返す。"""
        return _FEMALE_LABELS[self][1]


_FEMALE_LABELS: dict[FemaleWorkersProportion, tuple[str, str]] = {
    FemaleWorkersProportion.割合20以下: ("～20%", "20% or less"),
    FemaleWorkersProportion.割合21から40: ("21%～40%", "21%-40%"),
    FemaleWorkersProportion.割合41から60: ("41%～60%", "41%-60%"),
    FemaleWorkersProportion.割合61以上: ("61%～", "61% or more"),
}



"""キャッシュテスト。"""

from __future__ import annotations

import time
from typing import TYPE_CHECKING

import pytest

from gbizinfo.cache import FileCache
from gbizinfo.config import CacheMode

if TYPE_CHECKING:
    from pathlib import Path


class TestFileCache:
    def test_hit_miss(self, tmp_path: Path):
        cache = FileCache(cache_dir=tmp_path, ttl_seconds=3600)
        key = "test-key"

        # miss
        assert cache.get(key=key, mode=CacheMode.READ_WRITE) is None

        # put
        cache.put(key=key, payload={"data": "hello"})

        # hit
        hit = cache.get(key=key, mode=CacheMode.READ_WRITE)
        assert hit is not None
        assert hit.payload["data"] == "hello"
        assert hit.stale is False

    def test_ttl_expiry(self, tmp_path: Path):
        cache = FileCache(cache_dir=tmp_path, ttl_seconds=0)
        key = "expire-key"
        cache.put(key=key, payload={"data": "old"})
        time.sleep(0.01)
        hit = cache.get(key=key, mode=CacheMode.READ_WRITE)
        assert hit is not None
        assert hit.stale is True

    def test_off_mode(self, tmp_path: Path):
        cache = FileCache(cache_dir=tmp_path, ttl_seconds=3600)
        key = "off-key"
        cache.put(key=key, payload={"data": "value"})
        assert cache.get(key=key, mode=CacheMode.OFF) is None

    def test_corrupt_file_quarantine(self, tmp_path: Path):
        cache = FileCache(cache_dir=tmp_path, ttl_seconds=3600)
        key = "corrupt-key"

        # Write a corrupt file
        import hashlib
        digest = hashlib.sha256(key.encode("utf-8")).hexdigest()
        corrupt_path = tmp_path / f"{digest}.json"
        corrupt_path.write_text("NOT JSON{{{", encoding="utf-8")

        # Get should return None and quarantine
        assert cache.get(key=key, mode=CacheMode.READ_WRITE) is None
        assert (tmp_path / ".broken" / corrupt_path.name).exists()

    def test_no_cache_dir(self):
        cache = FileCache(cache_dir=None, ttl_seconds=3600)
        assert cache.get(key="any", mode=CacheMode.READ_WRITE) is None
        cache.put(key="any", payload={"data": "ignored"})  # no-op


class TestMakeKey:
    def test_token_not_in_key(self):
        """キーに token が含まれないこと。"""
        key1 = FileCache.make_key(method="GET", path="/v2/hojin", params={"name": "test"})
        key2 = FileCache.make_key(method="GET", path="/v2/hojin", params={"name": "test"})
        assert key1 == key2

    def test_different_params_different_keys(self):
        key1 = FileCache.make_key(method="GET", path="/v2/hojin", params={"name": "a"})
        key2 = FileCache.make_key(method="GET", path="/v2/hojin", params={"name": "b"})
        assert key1 != key2

    def test_metadata_changes_key(self):
        key1 = FileCache.make_key(method="GET", path="/v2/hojin", params={})
        key2 = FileCache.make_key(method="GET", path="/v2/hojin", params={"metadata_flg": "true"})
        assert key1 != key2


@pytest.mark.anyio()
class TestAsyncCache:
    async def test_async_hit_miss(self, tmp_path: Path):
        cache = FileCache(cache_dir=tmp_path, ttl_seconds=3600)
        key = "async-test"

        assert await cache.aget(key=key, mode=CacheMode.READ_WRITE) is None

        await cache.aput(key=key, payload={"async": True})

        hit = await cache.aget(key=key, mode=CacheMode.READ_WRITE)
        assert hit is not None
        assert hit.payload["async"] is True



"""法人種別の列挙型モジュール。

gBizINFO API の法人種別コードに対応する StrEnum を定義する。
"""

from __future__ import annotations

from enum import StrEnum


class CorporateType(StrEnum):
    """法人種別の列挙型。

    gBizINFO API で使用される法人種別コードを定義する。
    検索時に corporate_type パラメータとして指定する。
    """

    国の機関 = "101"
    地方公共団体 = "201"
    株式会社 = "301"
    有限会社 = "302"
    合名会社 = "303"
    合資会社 = "304"
    合同会社 = "305"
    その他の設立登記法人 = "399"
    外国会社等 = "401"
    その他 = "499"


# 英語 alias
NATIONAL_AGENCY = CorporateType.国の機関
LOCAL_GOVERNMENT = CorporateType.地方公共団体
KABUSHIKI_KAISHA = CorporateType.株式会社
YUGEN_KAISHA = CorporateType.有限会社
GOMEI_KAISHA = CorporateType.合名会社
GOSHI_KAISHA = CorporateType.合資会社
GODO_KAISHA = CorporateType.合同会社
OTHER_REGISTERED = CorporateType.その他の設立登記法人
FOREIGN_COMPANY = CorporateType.外国会社等
OTHER = CorporateType.その他



"""都道府県（JIS X 0401）の列挙型モジュール。

47都道府県をJIS X 0401コードで定義する。
"""

from __future__ import annotations

from enum import StrEnum


class Prefecture(StrEnum):
    """都道府県の列挙型（JIS X 0401 準拠）。

    各都道府県に JIS X 0401 の2桁コードを割り当てる。
    検索時に prefecture パラメータとして指定する。
    """

    北海道 = "01"
    青森県 = "02"
    岩手県 = "03"
    宮城県 = "04"
    秋田県 = "05"
    山形県 = "06"
    福島県 = "07"
    茨城県 = "08"
    栃木県 = "09"
    群馬県 = "10"
    埼玉県 = "11"
    千葉県 = "12"
    東京都 = "13"
    神奈川県 = "14"
    新潟県 = "15"
    富山県 = "16"
    石川県 = "17"
    福井県 = "18"
    山梨県 = "19"
    長野県 = "20"
    岐阜県 = "21"
    静岡県 = "22"
    愛知県 = "23"
    三重県 = "24"
    滋賀県 = "25"
    京都府 = "26"
    大阪府 = "27"
    兵庫県 = "28"
    奈良県 = "29"
    和歌山県 = "30"
    鳥取県 = "31"
    島根県 = "32"
    岡山県 = "33"
    広島県 = "34"
    山口県 = "35"
    徳島県 = "36"
    香川県 = "37"
    愛媛県 = "38"
    高知県 = "39"
    福岡県 = "40"
    佐賀県 = "41"
    長崎県 = "42"
    熊本県 = "43"
    大分県 = "44"
    宮崎県 = "45"
    鹿児島県 = "46"
    沖縄県 = "47"


# 英語 alias
HOKKAIDO = Prefecture.北海道
AOMORI = Prefecture.青森県
IWATE = Prefecture.岩手県
MIYAGI = Prefecture.宮城県
AKITA = Prefecture.秋田県
YAMAGATA = Prefecture.山形県
FUKUSHIMA = Prefecture.福島県
IBARAKI = Prefecture.茨城県
TOCHIGI = Prefecture.栃木県
GUNMA = Prefecture.群馬県
SAITAMA = Prefecture.埼玉県
CHIBA = Prefecture.千葉県
TOKYO = Prefecture.東京都
KANAGAWA = Prefecture.神奈川県
NIIGATA = Prefecture.新潟県
TOYAMA = Prefecture.富山県
ISHIKAWA = Prefecture.石川県
FUKUI = Prefecture.福井県
YAMANASHI = Prefecture.山梨県
NAGANO = Prefecture.長野県
GIFU = Prefecture.岐阜県
SHIZUOKA = Prefecture.静岡県
AICHI = Prefecture.愛知県
MIE = Prefecture.三重県
SHIGA = Prefecture.滋賀県
KYOTO = Prefecture.京都府
OSAKA = Prefecture.大阪府
HYOGO = Prefecture.兵庫県
NARA = Prefecture.奈良県
WAKAYAMA = Prefecture.和歌山県
TOTTORI = Prefecture.鳥取県
SHIMANE = Prefecture.島根県
OKAYAMA = Prefecture.岡山県
HIROSHIMA = Prefecture.広島県
YAMAGUCHI = Prefecture.山口県
TOKUSHIMA = Prefecture.徳島県
KAGAWA = Prefecture.香川県
EHIME = Prefecture.愛媛県
KOCHI = Prefecture.高知県
FUKUOKA = Prefecture.福岡県
SAGA = Prefecture.佐賀県
NAGASAKI = Prefecture.長崎県
KUMAMOTO = Prefecture.熊本県
OITA = Prefecture.大分県
MIYAZAKI = Prefecture.宮崎県
KAGOSHIMA = Prefecture.鹿児島県
OKINAWA = Prefecture.沖縄県



#!/usr/bin/env python3
"""openapi.json → _generated.py の自動生成スクリプト。"""

from __future__ import annotations

import subprocess
import sys
from pathlib import Path

ROOT = Path(__file__).resolve().parent.parent
INPUT = ROOT / "openapi.json"
OUTPUT = ROOT / "src" / "gbizinfo" / "models" / "_generated.py"


def main() -> None:
    cmd = [
        sys.executable,
        "-m",
        "datamodel_code_generator",
        "--input",
        str(INPUT),
        "--output",
        str(OUTPUT),
        "--output-model-type",
        "pydantic_v2.BaseModel",
        "--input-file-type",
        "openapi",
        "--target-python-version",
        "3.12",
        "--use-standard-collections",
        "--use-union-operator",
        "--field-constraints",
        "--capitalise-enum-members",
        "--use-field-description",
        "--strict-nullable",
        "--base-class",
        "gbizinfo.models._base.BaseModel",
    ]
    print(f"Running: {' '.join(cmd)}")
    result = subprocess.run(cmd, capture_output=True, text=True)
    if result.returncode != 0:
        print("STDOUT:", result.stdout)
        print("STDERR:", result.stderr)
        sys.exit(result.returncode)

    # OpenAPI spec の kana パターンは ー（長音符）を含まず実データと不整合のため除去
    content = OUTPUT.read_text(encoding="utf-8")
    content = content.replace(
        "pattern='^[ァ-ヶ]*$'",
        "",
    )
    # 空引数のカンマ整理 (e.g. "Field(None, max_length=200, min_length=0, )")
    import re
    content = re.sub(r",\s*\)", ")", content)
    OUTPUT.write_text(content, encoding="utf-8")

    # ハイフン alias 検証
    required_aliases = ["hojin-infos", "corporation-info", "meta-data"]
    for alias in required_aliases:
        if alias not in content:
            print(f"WARNING: alias '{alias}' not found in generated output")
        else:
            print(f"OK: alias '{alias}' found")

    print(f"Generated: {OUTPUT}")


if __name__ == "__main__":
    main()



"""営業品目コードおよび資格の種類の列挙型モジュール。

全省庁統一資格の営業品目コードと資格の種類を定義する。
"""

from __future__ import annotations

from enum import StrEnum


class QualificationType(StrEnum):
    """全省庁統一資格の種類の列挙型。

    物品の製造・販売、役務の提供等、物品の買受けの4種類を定義する。
    """

    物品の製造 = "manufacturing"
    物品の販売 = "sales"
    役務の提供等 = "services"
    物品の買受け = "purchase"


class BusinessItem(StrEnum):
    """全省庁統一資格の営業品目コードの列挙型。

    物品の製造(101-129)、物品の販売(201-229)、役務の提供等(301-315)、
    物品の買受け(401-402) の各品目を定義する。
    ``qualification_type`` プロパティで属する資格の種類を取得できる。
    """

    # 物品の製造 (101-129)
    衣服その他繊維製品類_製造 = "101"
    ゴム皮革プラスチック製品類_製造 = "102"
    窯業土石製品類_製造 = "103"
    非鉄金属金属製品類_製造 = "104"
    フォーム印刷_製造 = "105"
    その他印刷類_製造 = "106"
    図書類_製造 = "107"
    電子出版物類_製造 = "108"
    紙紙加工品類_製造 = "109"
    車両類_製造 = "110"
    その他輸送搬送機械器具類_製造 = "111"
    船舶類_製造 = "112"
    燃料類_製造 = "113"
    家具什器類_製造 = "114"
    一般産業用機器類_製造 = "115"
    電気通信用機器類_製造 = "116"
    電子計算機類_製造 = "117"
    精密機器類_製造 = "118"
    医療用機器類_製造 = "119"
    事務用機器類_製造 = "120"
    その他機器類_製造 = "121"
    医薬品医療用品類_製造 = "122"
    事務用品類_製造 = "123"
    土木建設建築材料_製造 = "124"
    警察用装備品類_製造 = "127"
    防衛用装備品類_製造 = "128"
    その他_製造 = "129"

    # 物品の販売 (201-229)
    衣服その他繊維製品類_販売 = "201"
    ゴム皮革プラスチック製品類_販売 = "202"
    窯業土石製品類_販売 = "203"
    非鉄金属金属製品類_販売 = "204"
    フォーム印刷_販売 = "205"
    その他印刷類_販売 = "206"
    図書類_販売 = "207"
    電子出版物類_販売 = "208"
    紙紙加工品類_販売 = "209"
    車両類_販売 = "210"
    その他輸送搬送機械器具類_販売 = "211"
    船舶類_販売 = "212"
    燃料類_販売 = "213"
    家具什器類_販売 = "214"
    一般産業用機器類_販売 = "215"
    電気通信用機器類_販売 = "216"
    電子計算機類_販売 = "217"
    精密機器類_販売 = "218"
    医療用機器類_販売 = "219"
    事務用機器類_販売 = "220"
    その他機器類_販売 = "221"
    医薬品医療用品類_販売 = "222"
    事務用品類_販売 = "223"
    土木建設建築材料_販売 = "224"
    警察用装備品類_販売 = "227"
    防衛用装備品類_販売 = "228"
    その他_販売 = "229"

    # 役務の提供等 (301-315)
    広告宣伝 = "301"
    写真製図 = "302"
    調査研究 = "303"
    情報処理 = "304"
    翻訳通訳速記 = "305"
    ソフトウェア開発 = "306"
    会場等の借り上げ = "307"
    賃貸借 = "308"
    建物管理等各種保守管理 = "309"
    運送 = "310"
    車両整備 = "311"
    船舶整備 = "312"
    電子出版 = "313"
    防衛用装備品類の整備 = "314"
    その他_役務 = "315"

    # 物品の買受け (401-402)
    立木竹 = "401"
    その他_買受け = "402"

    @property
    def qualification_type(self) -> QualificationType:
        """この品目が属する資格の種類を返す。

        Returns:
            営業品目コードに対応する資格の種類。
        """
        code = int(self.value)
        if code < 200:
            return QualificationType.物品の製造
        if code < 300:
            return QualificationType.物品の販売
        if code < 400:
            return QualificationType.役務の提供等
        return QualificationType.物品の買受け



"""Enum 値の網羅テスト。"""

from __future__ import annotations

from gbizinfo.enums import (
    AverageAge,
    AverageContinuousServiceYears,
    BusinessItem,
    CorporateType,
    DesignClassification,
    FemaleWorkersProportion,
    Ministry,
    MonthAverageOvertimeHours,
    PatentClassification,
    PatentType,
    Prefecture,
    QualificationType,
    Region,
    Source,
    TrademarkClassification,
)


class TestCorporateType:
    def test_member_count(self):
        assert len(CorporateType) == 10

    def test_values(self):
        assert CorporateType.株式会社.value == "301"
        assert CorporateType.国の機関.value == "101"
        assert CorporateType.その他.value == "499"

    def test_english_alias(self):
        from gbizinfo.enums.corporate_type import KABUSHIKI_KAISHA
        assert KABUSHIKI_KAISHA is CorporateType.株式会社


class TestPrefecture:
    def test_member_count(self):
        assert len(Prefecture) == 47

    def test_values(self):
        assert Prefecture.北海道.value == "01"
        assert Prefecture.東京都.value == "13"
        assert Prefecture.沖縄県.value == "47"

    def test_english_alias(self):
        from gbizinfo.enums.prefecture import TOKYO
        assert TOKYO is Prefecture.東京都


class TestRegion:
    def test_member_count(self):
        assert len(Region) == 10

    def test_prefectures_property(self):
        assert Prefecture.東京都 in Region.関東.prefectures
        assert Prefecture.北海道 in Region.北海道.prefectures
        assert Prefecture.沖縄県 in Region.沖縄.prefectures

    def test_all_prefectures_covered(self):
        all_prefs = set()
        for region in Region:
            all_prefs.update(region.prefectures)
        assert len(all_prefs) == 47


class TestSource:
    def test_member_count(self):
        assert len(Source) == 6

    def test_values(self):
        assert Source.調達.value == "1"
        assert Source.財務.value == "6"


class TestMinistry:
    def test_member_count(self):
        assert len(Ministry) == 49

    def test_values(self):
        assert Ministry.国税庁.value == "1"
        assert Ministry.こども家庭庁.value == "49"
        assert Ministry.国立研究開発法人新エネルギー産業技術総合開発機構.value == "30"


class TestBusinessItem:
    def test_member_count(self):
        """CODE.md の全56項目（製造27+販売27+役務15+買受2 = 71ではなく、欠番含め正確な数）。"""
        # 製造: 101-124, 127-129 = 27
        # 販売: 201-224, 227-229 = 27
        # 役務: 301-315 = 15
        # 買受: 401-402 = 2
        assert len(BusinessItem) == 71

    def test_qualification_type(self):
        assert BusinessItem.衣服その他繊維製品類_製造.qualification_type == QualificationType.物品の製造
        assert BusinessItem.衣服その他繊維製品類_販売.qualification_type == QualificationType.物品の販売
        assert BusinessItem.広告宣伝.qualification_type == QualificationType.役務の提供等
        assert BusinessItem.立木竹.qualification_type == QualificationType.物品の買受け


class TestWorkplace:
    def test_average_age(self):
        assert len(AverageAge) == 4
        assert AverageAge.歳30以下.value == "A"
        assert AverageAge.歳30以下.label == "～30歳"
        assert AverageAge.歳30以下.label_en == "30 or under"

    def test_service_years(self):
        assert len(AverageContinuousServiceYears) == 4

    def test_overtime(self):
        assert len(MonthAverageOvertimeHours) == 3
        assert MonthAverageOvertimeHours.時間40以上.value == "C"

    def test_female_proportion(self):
        assert len(FemaleWorkersProportion) == 4


class TestPatent:
    def test_patent_classification_count(self):
        assert len(PatentClassification) == 131

    def test_patent_description(self):
        assert PatentClassification.農業_林業_畜産.description_ja == "農業; 林業; 畜産; 狩猟; 捕獲; 漁業"
        assert "Agriculture" in PatentClassification.農業_林業_畜産.description_en

    def test_design_classification_count(self):
        assert len(DesignClassification) == 85

    def test_design_description(self):
        assert DesignClassification.製造食品及び嗜好品.description_ja == "製造食品及び嗜好品"
        assert DesignClassification.車両.description_ja == "車両"
        assert DesignClassification.ソフトウェア.description_ja == "ソフトウェア"

    def test_trademark_classification_count(self):
        assert len(TrademarkClassification) == 45

    def test_trademark_description(self):
        assert TrademarkClassification.化学品.description_ja == "工業用、科学用又は農業用の化学品"
        assert TrademarkClassification.電気通信.description_ja == "電気通信"
        assert TrademarkClassification.冠婚葬祭_警備_法律.description_ja.startswith("冠婚葬祭")

    def test_patent_type(self):
        assert len(PatentType) == 3
        assert PatentType.特許.value == "特許"





"""クライアント設定値の定義モジュール。

キャッシュ設定、リトライ設定、クライアント共通設定など、
API クライアントの動作に関わる定数とデータクラスを定義する。
"""

from __future__ import annotations

from dataclasses import dataclass, field
from enum import StrEnum
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from pathlib import Path

DEFAULT_BASE_URL = "https://api.info.gbiz.go.jp/hojin"
MAX_PAGE = 10
MAX_LIMIT = 5000
MAX_TOTAL_RECORDS = MAX_PAGE * MAX_LIMIT  # 50,000


class CacheMode(StrEnum):
    """キャッシュモードの列挙型。

    Attributes:
        OFF: キャッシュ無効。
        READ: 読み取りのみ（既存キャッシュを利用するが書き込まない）。
        READ_WRITE: 読み書き両対応（通常のキャッシュ動作）。
        FORCE_REFRESH: キャッシュを無視してリクエストし、結果を書き込む。
    """

    OFF = "off"
    READ = "read"
    READ_WRITE = "read_write"
    FORCE_REFRESH = "force_refresh"


@dataclass(slots=True)
class RetryConfig:
    """リトライ（再試行）設定。

    Attributes:
        max_attempts: 最大リトライ回数。
        base_delay: バックオフの基本待機秒数。
        cap_delay: バックオフの上限待機秒数。
    """

    max_attempts: int = 5
    base_delay: float = 0.5
    cap_delay: float = 8.0


@dataclass(slots=True)
class CacheConfig:
    """キャッシュ設定。

    Attributes:
        mode: キャッシュモード。
        dir: キャッシュファイルの保存ディレクトリ。None でキャッシュ無効。
        ttl_seconds: キャッシュの有効期間（秒）。デフォルトは24時間。
    """

    mode: CacheMode = CacheMode.OFF
    dir: Path | None = None
    ttl_seconds: int = 24 * 60 * 60


@dataclass(slots=True)
class ClientConfig:
    """クライアント共通設定。

    Attributes:
        base_url: API のベース URL。
        timeout: HTTP タイムアウト秒数。
        user_agent: User-Agent ヘッダ値。
        rate_limit_per_sec: 1秒あたりの最大リクエスト数。
        cache: キャッシュ設定。
    """

    base_url: str = DEFAULT_BASE_URL
    timeout: float = 30.0
    user_agent: str = ""
    rate_limit_per_sec: float = 1.0
    cache: CacheConfig = field(default_factory=CacheConfig)



"""レスポンスラッパーモデルおよびフラット化拡張モジュール。

API レスポンスのラッパークラスと、ネストされた Pydantic モデルを
フラットな辞書に変換する ``to_flat_dict()`` 機能を提供する。
"""

from __future__ import annotations

import json
from dataclasses import dataclass, field
from typing import Any, Literal

from pydantic import BaseModel as PydanticBaseModel

from gbizinfo.models._generated import HojinInfoSearchV2, HojinInfoV2

ListStrategy = Literal["count", "first", "json", "explode"]


def _flatten(
    obj: Any,
    *,
    prefix: str,
    sep: str,
    lists: ListStrategy,
    max_items: int,
    result: dict[str, Any],
) -> None:
    """ネストされたオブジェクトを再帰的にフラット化する。

    Pydantic モデルのフィールドを辿り、プレフィックス付きの
    キーで result 辞書に書き込む。

    Args:
        obj: フラット化対象のオブジェクト。
        prefix: 現在のキーのプレフィックス。
        sep: プレフィックスの区切り文字。
        lists: リスト型フィールドの処理戦略。
        max_items: ``"explode"`` 戦略時の最大展開数。
        result: 結果を格納する辞書（破壊的に更新される）。
    """
    if obj is None:
        return

    if isinstance(obj, PydanticBaseModel):
        for field_name in type(obj).model_fields:
            value = getattr(obj, field_name, None)
            key = f"{prefix}{sep}{field_name}" if prefix else field_name
            _flatten(value, prefix=key, sep=sep, lists=lists, max_items=max_items, result=result)
        return

    if isinstance(obj, list):
        if lists == "count":
            result[f"{prefix}_count"] = len(obj)
        elif lists == "first":
            if obj:
                _flatten(obj[0], prefix=prefix, sep=sep, lists=lists, max_items=max_items, result=result)
        elif lists == "json":
            items = []
            for item in obj:
                if isinstance(item, PydanticBaseModel):
                    items.append(item.model_dump(by_alias=False))
                else:
                    items.append(item)
            result[prefix] = json.dumps(items, ensure_ascii=False, default=str)
        elif lists == "explode":
            for i, item in enumerate(obj[:max_items]):
                idx_prefix = f"{prefix}_{i}"
                _flatten(item, prefix=idx_prefix, sep=sep, lists=lists, max_items=max_items, result=result)
        return

    result[prefix] = obj


class _FlatDictMixin:
    """``to_flat_dict()`` メソッドを提供する Mixin クラス。

    Pydantic モデルに組み込むことで、ネストされたフィールドを
    フラットな辞書に変換する機能を追加する。
    """

    def to_flat_dict(
        self,
        *,
        prefix_separator: str = "_",
        lists: ListStrategy = "count",
        max_items: int = 10,
    ) -> dict[str, Any]:
        """ネストされたモデルをフラット化した辞書を返す。

        Args:
            prefix_separator: ネストしたフィールド名の区切り文字。
            lists: リスト型フィールドの処理戦略。
                ``"count"`` で件数のみ、``"first"`` で先頭要素のみ、
                ``"json"`` で JSON 文字列化、``"explode"`` でインデックス付き展開。
            max_items: ``"explode"`` 時の最大展開数。

        Returns:
            フラット化されたフィールド名と値の辞書。
        """
        result: dict[str, Any] = {}
        for field_name in type(self).model_fields:  # type: ignore[attr-defined]
            value = getattr(self, field_name, None)
            _flatten(
                value,
                prefix=field_name,
                sep=prefix_separator,
                lists=lists,
                max_items=max_items,
                result=result,
            )
        return result


class HojinInfo(_FlatDictMixin, HojinInfoV2):
    """法人情報（拡張）。

    ``HojinInfoV2`` に ``to_flat_dict()`` メソッドを追加した拡張モデル。
    """


class HojinInfoSearch(_FlatDictMixin, HojinInfoSearchV2):
    """法人検索情報（拡張）。

    ``HojinInfoSearchV2`` に ``to_flat_dict()`` メソッドを追加した拡張モデル。
    """


@dataclass
class SearchResult:
    """検索結果（/v2/hojin 系）を格納するデータクラス。

    Attributes:
        items: 検索にヒットした法人情報のリスト。
    """

    items: list[HojinInfoSearch] = field(default_factory=list)

    def to_flat_dicts(
        self,
        *,
        lists: ListStrategy = "count",
        max_items: int = 10,
    ) -> list[dict[str, Any]]:
        """全件をフラット化した辞書のリストを返す。

        Args:
            lists: リスト型フィールドの処理戦略。
            max_items: ``"explode"`` 時の最大展開数。

        Returns:
            各法人情報をフラット化した辞書のリスト。
        """
        return [item.to_flat_dict(lists=lists, max_items=max_items) for item in self.items]


@dataclass
class UpdateResult:
    """差分更新結果（/v2/hojin/updateInfo 系）を格納するデータクラス。

    Attributes:
        items: 更新された法人情報のリスト。
        total_count: 総件数。
        total_page: 総ページ数。
        page_number: 現在のページ番号。
    """

    items: list[HojinInfo] = field(default_factory=list)
    total_count: int = 0
    total_page: int = 0
    page_number: int = 0

    def to_flat_dicts(
        self,
        *,
        lists: ListStrategy = "count",
        max_items: int = 10,
    ) -> list[dict[str, Any]]:
        """全件をフラット化した辞書のリストを返す。

        Args:
            lists: リスト型フィールドの処理戦略。
            max_items: ``"explode"`` 時の最大展開数。

        Returns:
            各法人情報をフラット化した辞書のリスト。
        """
        return [item.to_flat_dict(lists=lists, max_items=max_items) for item in self.items]



"""ページネーション処理モジュール。

検索結果および差分更新結果の透過的なページネーションを提供する。
同期・非同期の両方に対応したジェネレータ関数を含む。
"""

from __future__ import annotations

from typing import TYPE_CHECKING, Any

from gbizinfo._logging import logger
from gbizinfo.config import MAX_PAGE, MAX_TOTAL_RECORDS
from gbizinfo.errors import PaginationLimitExceededError

if TYPE_CHECKING:
    from collections.abc import AsyncIterator, Iterator
    from datetime import date

    from gbizinfo.client import AsyncGbizClient, GbizClient
    from gbizinfo.models.responses import HojinInfo, HojinInfoSearch


def paginate_search_sync(
    client: GbizClient,
    params: dict[str, Any],
    *,
    start_page: int = 1,
    limit: int = 1000,
) -> Iterator[HojinInfoSearch]:
    """検索結果を透過的にページネーションする（同期版）。

    全ページを自動で取得し、各アイテムを順に yield する。
    ページネーション上限に達した場合は PaginationLimitExceededError を送出する。

    Args:
        client: 同期 gBizINFO クライアント。
        params: 検索パラメータの辞書。
        start_page: 開始ページ番号。
        limit: 1ページあたりの取得件数。

    Yields:
        検索結果の法人情報。

    Raises:
        PaginationLimitExceededError: ページネーション上限に到達した場合。
    """
    page = start_page
    while True:
        if page > MAX_PAGE:
            raise PaginationLimitExceededError(
                f"検索結果がページネーション上限（{MAX_PAGE}ページ / {MAX_TOTAL_RECORDS:,}件）に到達しました。"
                " 検索条件を絞り込む（都道府県・法人種別等の追加）か、limit を調整してください。",
                max_retrievable=MAX_TOTAL_RECORDS,
            )
        logger.debug("Fetching page %d/%d", page, MAX_PAGE)
        result = client.search(**params, page=page, limit=limit)
        yield from result.items
        if len(result.items) < limit:
            break
        page += 1


async def paginate_search_async(
    client: AsyncGbizClient,
    params: dict[str, Any],
    *,
    start_page: int = 1,
    limit: int = 1000,
) -> AsyncIterator[HojinInfoSearch]:
    """検索結果を透過的にページネーションする（非同期版）。

    全ページを自動で取得し、各アイテムを順に yield する。
    ページネーション上限に達した場合は PaginationLimitExceededError を送出する。

    Args:
        client: 非同期 gBizINFO クライアント。
        params: 検索パラメータの辞書。
        start_page: 開始ページ番号。
        limit: 1ページあたりの取得件数。

    Yields:
        検索結果の法人情報。

    Raises:
        PaginationLimitExceededError: ページネーション上限に到達した場合。
    """
    page = start_page
    while True:
        if page > MAX_PAGE:
            raise PaginationLimitExceededError(
                f"検索結果がページネーション上限（{MAX_PAGE}ページ / {MAX_TOTAL_RECORDS:,}件）に到達しました。"
                " 検索条件を絞り込む（都道府県・法人種別等の追加）か、limit を調整してください。",
                max_retrievable=MAX_TOTAL_RECORDS,
            )
        logger.debug("Fetching page %d/%d", page, MAX_PAGE)
        result = await client.search(**params, page=page, limit=limit)
        for item in result.items:
            yield item
        if len(result.items) < limit:
            break
        page += 1


def paginate_update_sync(
    client: GbizClient,
    *,
    from_date: date,
    to_date: date,
    metadata_flg: bool = False,
) -> Iterator[HojinInfo]:
    """差分更新結果を透過的にページネーションする（同期版）。

    全ページを自動で取得し、各アイテムを順に yield する。

    Args:
        client: 同期 gBizINFO クライアント。
        from_date: 更新期間の開始日。
        to_date: 更新期間の終了日。
        metadata_flg: メタデータを含めるかどうか。

    Yields:
        更新された法人情報。

    Raises:
        PaginationLimitExceededError: ページネーション上限に到達した場合。
    """
    page = 1
    while True:
        if page > MAX_PAGE:
            raise PaginationLimitExceededError(
                f"差分更新結果がページネーション上限（{MAX_PAGE}ページ）に到達しました。"
                " 期間を短くして分割取得してください。",
                max_retrievable=MAX_TOTAL_RECORDS,
            )
        logger.debug("Fetching update page %d", page)
        result = client.get_update_info(
            from_date=from_date, to_date=to_date, page=page, metadata_flg=metadata_flg
        )
        yield from result.items
        if page >= result.total_page:
            break
        page += 1


async def paginate_update_async(
    client: AsyncGbizClient,
    *,
    from_date: date,
    to_date: date,
    metadata_flg: bool = False,
) -> AsyncIterator[HojinInfo]:
    """差分更新結果を透過的にページネーションする（非同期版）。

    全ページを自動で取得し、各アイテムを順に yield する。

    Args:
        client: 非同期 gBizINFO クライアント。
        from_date: 更新期間の開始日。
        to_date: 更新期間の終了日。
        metadata_flg: メタデータを含めるかどうか。

    Yields:
        更新された法人情報。

    Raises:
        PaginationLimitExceededError: ページネーション上限に到達した場合。
    """
    page = 1
    while True:
        if page > MAX_PAGE:
            raise PaginationLimitExceededError(
                f"差分更新結果がページネーション上限（{MAX_PAGE}ページ）に到達しました。"
                " 期間を短くして分割取得してください。",
                max_retrievable=MAX_TOTAL_RECORDS,
            )
        logger.debug("Fetching update page %d", page)
        result = await client.get_update_info(
            from_date=from_date, to_date=to_date, page=page, metadata_flg=metadata_flg
        )
        for item in result.items:
            yield item
        if page >= result.total_page:
            break
        page += 1



"""法人番号バリデーションテスト。"""

from __future__ import annotations

from datetime import date

import pytest

from gbizinfo.errors import GbizCorporateNumberError
from gbizinfo.validation import format_date, validate_corporate_number


class TestValidateCorporateNumber:
    """法人番号バリデーション。"""

    def test_valid_number(self):
        """正しい法人番号はそのまま返る。"""
        # 1180001015846: body 180001015846 → check=1
        assert validate_corporate_number("1180001015846") == "1180001015846"

    def test_valid_number_national_tax(self):
        """国税庁の法人番号。"""
        assert validate_corporate_number("7000012050002") == "7000012050002"

    def test_too_short(self):
        """12桁以下は拒否。"""
        with pytest.raises(GbizCorporateNumberError, match="13桁"):
            validate_corporate_number("123456789012")

    def test_too_long(self):
        """14桁以上は拒否。"""
        with pytest.raises(GbizCorporateNumberError, match="13桁"):
            validate_corporate_number("12345678901234")

    def test_non_digit(self):
        """数字以外は拒否。"""
        with pytest.raises(GbizCorporateNumberError, match="13桁"):
            validate_corporate_number("123456789012a")

    def test_bad_check_digit(self):
        """チェックデジットが不正な場合。"""
        # 2180001015846: 正しいチェックデジットは 1 なので 2 は不正
        with pytest.raises(GbizCorporateNumberError, match="チェックデジット"):
            validate_corporate_number("2180001015846")

    def test_empty(self):
        """空文字列は拒否。"""
        with pytest.raises(GbizCorporateNumberError):
            validate_corporate_number("")


class TestFormatDate:
    """日付フォーマット。"""

    def test_format(self):
        assert format_date(date(2024, 1, 15)) == "20240115"

    def test_format_padded(self):
        assert format_date(date(2024, 3, 5)) == "20240305"



"""全 Enum 再エクスポートモジュール。

gBizINFO API で使用する全ての列挙型をこのパッケージから一括インポートできる。
"""

from gbizinfo.enums.business_item import BusinessItem, QualificationType
from gbizinfo.enums.corporate_type import CorporateType
from gbizinfo.enums.ministry import Ministry
from gbizinfo.enums.patent import (
    DesignClassification,
    PatentClassification,
    PatentType,
    TrademarkClassification,
)
from gbizinfo.enums.prefecture import Prefecture
from gbizinfo.enums.region import Region
from gbizinfo.enums.source import Source
from gbizinfo.enums.workplace import (
    AverageAge,
    AverageContinuousServiceYears,
    FemaleWorkersProportion,
    MonthAverageOvertimeHours,
)

__all__ = [
    "AverageAge",
    "AverageContinuousServiceYears",
    "BusinessItem",
    "CorporateType",
    "DesignClassification",
    "FemaleWorkersProportion",
    "Ministry",
    "MonthAverageOvertimeHours",
    "PatentClassification",
    "PatentType",
    "Prefecture",
    "QualificationType",
    "Region",
    "Source",
    "TrademarkClassification",
]



"""地域対応表（gBizINFO 内部コード）モジュール。

日本の地域区分と各地域に含まれる都道府県の対応を定義する。
"""

from __future__ import annotations

from enum import StrEnum

from gbizinfo.enums.prefecture import Prefecture


class Region(StrEnum):
    """地域の列挙型（gBizINFO 内部コード）。

    日本の10地域区分を定義する。各地域が含む都道府県は
    ``prefectures`` プロパティで取得できる。
    """

    北海道 = "1"
    東北 = "2"
    関東 = "3"
    北陸 = "4"
    中部 = "5"
    近畿 = "6"
    中国 = "7"
    四国 = "8"
    九州 = "9"
    沖縄 = "10"

    @property
    def prefectures(self) -> tuple[Prefecture, ...]:
        """この地域に含まれる都道府県のタプルを返す。

        Returns:
            都道府県の列挙値のタプル。
        """
        return _REGION_PREFECTURES[self]


_REGION_PREFECTURES: dict[Region, tuple[Prefecture, ...]] = {
    Region.北海道: (Prefecture.北海道,),
    Region.東北: (
        Prefecture.青森県,
        Prefecture.岩手県,
        Prefecture.宮城県,
        Prefecture.秋田県,
        Prefecture.山形県,
        Prefecture.福島県,
    ),
    Region.関東: (
        Prefecture.茨城県,
        Prefecture.栃木県,
        Prefecture.群馬県,
        Prefecture.埼玉県,
        Prefecture.千葉県,
        Prefecture.東京都,
        Prefecture.神奈川県,
    ),
    Region.北陸: (
        Prefecture.新潟県,
        Prefecture.富山県,
        Prefecture.石川県,
        Prefecture.福井県,
    ),
    Region.中部: (
        Prefecture.山梨県,
        Prefecture.長野県,
        Prefecture.岐阜県,
        Prefecture.静岡県,
        Prefecture.愛知県,
        Prefecture.三重県,
    ),
    Region.近畿: (
        Prefecture.滋賀県,
        Prefecture.京都府,
        Prefecture.大阪府,
        Prefecture.兵庫県,
        Prefecture.奈良県,
        Prefecture.和歌山県,
    ),
    Region.中国: (
        Prefecture.鳥取県,
        Prefecture.島根県,
        Prefecture.岡山県,
        Prefecture.広島県,
        Prefecture.山口県,
    ),
    Region.四国: (
        Prefecture.徳島県,
        Prefecture.香川県,
        Prefecture.愛媛県,
        Prefecture.高知県,
    ),
    Region.九州: (
        Prefecture.福岡県,
        Prefecture.佐賀県,
        Prefecture.長崎県,
        Prefecture.熊本県,
        Prefecture.大分県,
        Prefecture.宮崎県,
        Prefecture.鹿児島県,
    ),
    Region.沖縄: (Prefecture.沖縄県,),
}


# 英語 alias
HOKKAIDO = Region.北海道
TOHOKU = Region.東北
KANTO = Region.関東
HOKURIKU = Region.北陸
CHUBU = Region.中部
KINKI = Region.近畿
CHUGOKU = Region.中国
SHIKOKU = Region.四国
KYUSHU = Region.九州
OKINAWA = Region.沖縄



"""クライアント統合テスト。"""

from __future__ import annotations

import inspect
from datetime import date

import httpx
import pytest

from gbizinfo import AsyncGbizClient, GbizClient
from gbizinfo.client import _PARAM_MAPPING, SearchParams, _build_query_params, _convert_param_value
from gbizinfo.enums import CorporateType, Prefecture
from gbizinfo.errors import GbizValidationError
from gbizinfo.models.responses import HojinInfo, HojinInfoSearch

BASE_URL = "https://api.info.gbiz.go.jp/hojin"


class TestParamMapping:
    """_PARAM_MAPPING テスト。"""

    def test_all_keys_present(self):
        """全パラメータが定義されている。"""
        expected_keys = {
            "corporate_number", "name", "exist_flg", "corporate_type", "prefecture",
            "city", "capital_stock_from", "capital_stock_to", "employee_number_from",
            "employee_number_to", "founded_year", "net_sales_from", "net_sales_to",
            "total_assets_from", "total_assets_to", "average_continuous_service_years",
            "average_age", "month_average_predetermined_overtime_hours",
            "female_workers_proportion", "patent", "procurement",
            "procurement_amount_from", "procurement_amount_to", "subsidy",
            "subsidy_amount_from", "subsidy_amount_to", "certification", "ministry",
            "source", "page", "limit", "metadata_flg",
        }
        assert set(_PARAM_MAPPING.keys()) == expected_keys

    def test_net_sales_mapping(self):
        """短縮名がAPIパラメータ名にマップされる。"""
        assert _PARAM_MAPPING["net_sales_from"] == "net_sales_summary_of_business_results_from"
        assert _PARAM_MAPPING["net_sales_to"] == "net_sales_summary_of_business_results_to"

    def test_total_assets_mapping(self):
        assert _PARAM_MAPPING["total_assets_from"] == "total_assets_summary_of_business_results_from"

    def test_search_params_match_search_args(self):
        """search() の引数名が _PARAM_MAPPING / SearchParams と一致する。"""
        sig = inspect.signature(GbizClient.search)
        search_params = {p for p in sig.parameters if p != "self"}
        assert search_params == set(_PARAM_MAPPING.keys())
        assert search_params == set(SearchParams.__annotations__.keys())

    def test_async_search_params_match(self):
        """AsyncGbizClient.search() も同じ引数を持つ。"""
        sync_sig = inspect.signature(GbizClient.search)
        async_sig = inspect.signature(AsyncGbizClient.search)
        sync_params = {p for p in sync_sig.parameters if p != "self"}
        async_params = {p for p in async_sig.parameters if p != "self"}
        assert sync_params == async_params


class TestParamConversion:
    """パラメータ変換テスト。"""

    def test_bool_conversion(self):
        assert _convert_param_value("exist_flg", True) == "true"
        assert _convert_param_value("exist_flg", False) == "false"

    def test_enum_conversion(self):
        assert _convert_param_value("prefecture", Prefecture.東京都) == "13"

    def test_int_conversion(self):
        assert _convert_param_value("capital_stock_from", 1000000) == "1000000"

    def test_sequence_enum_conversion(self):
        result = _convert_param_value(
            "corporate_type",
            [CorporateType.株式会社, CorporateType.合同会社]
        )
        assert result == "301,305"

    def test_build_query_params_skips_none(self):
        params = _build_query_params({"name": "test", "prefecture": None, "page": 1})
        assert "name" in params
        assert "prefecture" not in params


class TestTokenResolution:
    """api_token 解決テスト。"""

    def test_explicit_token(self, mock_api):
        client = GbizClient(api_token="explicit-token")
        assert client._api_token == "explicit-token"
        client.close()

    def test_env_fallback(self, mock_api, monkeypatch):
        monkeypatch.setenv("GBIZINFO_API_TOKEN", "env-token")
        client = GbizClient()
        assert client._api_token == "env-token"
        client.close()

    def test_no_token_raises(self, mock_api, monkeypatch):
        monkeypatch.delenv("GBIZINFO_API_TOKEN", raising=False)
        with pytest.raises(GbizValidationError, match="api_token"):
            GbizClient()


class TestSearch:
    """search() テスト。"""

    def test_basic_search(self, mock_api, client):
        mock_api.get("/v2/hojin").mock(
            return_value=httpx.Response(200, json={
                "hojin-infos": [
                    {"corporate_number": "1234567890123", "name": "テスト"},
                ],
                "id": "test",
                "message": "ok",
            })
        )
        result = client.search(name="テスト")
        assert len(result.items) == 1
        assert isinstance(result.items[0], HojinInfoSearch)

    def test_enum_params_converted(self, mock_api, client):
        mock_api.get("/v2/hojin").mock(
            return_value=httpx.Response(200, json={
                "hojin-infos": [],
                "id": "test",
                "message": "ok",
            })
        )
        client.search(prefecture=Prefecture.東京都, corporate_type=CorporateType.株式会社)
        request = mock_api.calls.last.request
        assert "prefecture=13" in str(request.url)
        assert "corporate_type=301" in str(request.url)


class TestGet:
    """get() テスト。"""

    def test_get_by_corporate_number(self, mock_api, client):
        mock_api.get("/v2/hojin/7000012050002").mock(
            return_value=httpx.Response(200, json={
                "hojin-infos": [
                    {"corporate_number": "7000012050002", "name": "国税庁"},
                ],
                "id": "test",
                "message": "ok",
            })
        )
        result = client.get("7000012050002")
        assert isinstance(result, HojinInfo)
        assert result.name == "国税庁"


class TestUpdateInfo:
    """差分更新テスト。"""

    def test_update_info(self, mock_api, client):
        mock_api.get("/v2/hojin/updateInfo").mock(
            return_value=httpx.Response(200, json={
                "hojin-infos": [
                    {"corporate_number": "1234567890123", "name": "更新テスト"},
                ],
                "id": "test",
                "message": "ok",
                "totalCount": "100",
                "totalPage": "1",
                "pageNumber": "1",
            })
        )
        result = client.get_update_info(
            from_date=date(2024, 1, 1),
            to_date=date(2024, 1, 31),
        )
        assert result.total_count == 100
        assert result.total_page == 1
        assert len(result.items) == 1


class TestContextManager:
    """コンテキストマネージャテスト。"""

    def test_sync_context_manager(self, mock_api):
        with GbizClient(api_token="test") as c:
            assert c is not None

    @pytest.mark.anyio()
    async def test_async_context_manager(self, mock_api):
        async with AsyncGbizClient(api_token="test") as c:
            assert c is not None



"""出典元の列挙型モジュール。

gBizINFO API の出典元コードに対応する StrEnum を定義する。
"""

from __future__ import annotations

from enum import StrEnum


class Source(StrEnum):
    """出典元の列挙型。

    gBizINFO API で使用される出典元コードを定義する。
    検索時に source パラメータとして指定する。
    """

    調達 = "1"
    表彰 = "2"
    届出認定 = "3"
    補助金 = "4"
    特許 = "5"
    財務 = "6"


# 英語 alias
PROCUREMENT = Source.調達
COMMENDATION = Source.表彰
CERTIFICATION = Source.届出認定
SUBSIDY = Source.補助金
PATENT = Source.特許
FINANCE = Source.財務



"""ページネーションテスト。"""

from __future__ import annotations

import httpx
import pytest

from gbizinfo.errors import PaginationLimitExceededError

BASE_URL = "https://api.info.gbiz.go.jp/hojin"


class TestPaginateSearch:
    """paginate_search テスト。"""

    def test_single_page(self, mock_api, client):
        """1ページ分の結果で終了。"""
        mock_api.get("/v2/hojin").mock(
            return_value=httpx.Response(200, json={
                "hojin-infos": [
                    {"corporate_number": f"{i:013d}", "name": f"法人{i}"}
                    for i in range(50)
                ],
                "id": "test",
                "message": "ok",
            })
        )
        items = list(client.paginate_search(name="test", limit=1000))
        assert len(items) == 50

    def test_multiple_pages(self, mock_api, client):
        """複数ページの透過的取得。"""
        call_count = 0

        def handler(request):
            nonlocal call_count
            call_count += 1
            if call_count < 3:
                items = [{"corporate_number": f"{i:013d}", "name": f"法人{i}"} for i in range(10)]
            else:
                items = [{"corporate_number": f"{i:013d}", "name": f"法人{i}"} for i in range(5)]
            return httpx.Response(200, json={
                "hojin-infos": items,
                "id": "test",
                "message": "ok",
            })

        mock_api.get("/v2/hojin").mock(side_effect=handler)
        items = list(client.paginate_search(name="test", limit=10))
        assert len(items) == 25  # 10 + 10 + 5
        assert call_count == 3

    def test_page_limit_exceeded(self, mock_api, client):
        """10ページ超過で PaginationLimitExceededError。"""
        # Always return full page
        mock_api.get("/v2/hojin").mock(
            return_value=httpx.Response(200, json={
                "hojin-infos": [
                    {"corporate_number": f"{i:013d}", "name": f"法人{i}"}
                    for i in range(1000)
                ],
                "id": "test",
                "message": "ok",
            })
        )
        with pytest.raises(PaginationLimitExceededError, match="上限"):
            list(client.paginate_search(name="test", limit=1000))

    def test_empty_result(self, mock_api, client):
        """空結果。"""
        mock_api.get("/v2/hojin").mock(
            return_value=httpx.Response(200, json={
                "hojin-infos": [],
                "id": "test",
                "message": "ok",
            })
        )
        items = list(client.paginate_search(name="nonexistent", limit=1000))
        assert len(items) == 0


class TestPaginateUpdateInfo:
    """paginate_update_info テスト。"""

    def test_uses_total_page(self, mock_api, client):
        """totalPage で終了判定。"""
        from datetime import date

        call_count = 0

        def handler(request):
            nonlocal call_count
            call_count += 1
            return httpx.Response(200, json={
                "hojin-infos": [
                    {"corporate_number": f"{i:013d}", "name": f"法人{i}"}
                    for i in range(3)
                ],
                "id": "test",
                "message": "ok",
                "totalCount": "6",
                "totalPage": "2",
                "pageNumber": str(call_count),
            })

        mock_api.get("/v2/hojin/updateInfo").mock(side_effect=handler)
        items = list(client.paginate_update_info(
            from_date=date(2024, 1, 1),
            to_date=date(2024, 1, 31),
        ))
        assert len(items) == 6
        assert call_count == 2



"""データモデル。"""

from gbizinfo.models._generated import (
    ApiError,
    CertificationInfoV2,
    ChildMetadata,
    CommendationInfoV2,
    CompatibilityOfChildcareAndWorkV2,
    CorporationInfo,
    FinanceV2,
    HojinInfoForChildV2,
    HojinInfoResponseForChildV2,
    HojinInfoResponseSearchV2,
    HojinInfoResponseV2,
    HojinInfoSearchV2,
    HojinInfoUpdateInfoResponseForChildV2,
    HojinInfoUpdateInfoResponseV2,
    HojinInfoV2,
    MajorShareholders,
    ManagementIndex,
    Metadata,
    PatentInfoV2,
    ProcurementInfoV2,
    SubsidyInfoV2,
    WomenActivityInfosV2,
    WorkplaceBaseInfosV2,
    WorkplaceInfoBeanV2,
)
from gbizinfo.models.responses import (
    HojinInfo,
    HojinInfoSearch,
    ListStrategy,
    SearchResult,
    UpdateResult,
)

__all__ = [
    "ApiError",
    "CertificationInfoV2",
    "ChildMetadata",
    "CommendationInfoV2",
    "CompatibilityOfChildcareAndWorkV2",
    "CorporationInfo",
    "FinanceV2",
    "HojinInfo",
    "HojinInfoForChildV2",
    "HojinInfoResponseForChildV2",
    "HojinInfoResponseSearchV2",
    "HojinInfoResponseV2",
    "HojinInfoSearch",
    "HojinInfoSearchV2",
    "HojinInfoUpdateInfoResponseForChildV2",
    "HojinInfoUpdateInfoResponseV2",
    "HojinInfoV2",
    "ListStrategy",
    "MajorShareholders",
    "ManagementIndex",
    "Metadata",
    "PatentInfoV2",
    "ProcurementInfoV2",
    "SearchResult",
    "SubsidyInfoV2",
    "UpdateResult",
    "WomenActivityInfosV2",
    "WorkplaceBaseInfosV2",
    "WorkplaceInfoBeanV2",
]



"""担当府省コードの列挙型モジュール。

gBizINFO API の担当府省コードに対応する StrEnum を定義する。
"""

from __future__ import annotations

from enum import StrEnum


class Ministry(StrEnum):
    """担当府省の列挙型。

    gBizINFO API で使用される担当府省コードを定義する。
    検索時に ministry パラメータとして指定する。
    """

    国税庁 = "1"
    会計検査院 = "2"
    内閣官房 = "3"
    人事院 = "4"
    内閣府 = "5"
    宮内庁 = "6"
    国家公安委員会 = "7"
    防衛省 = "8"
    金融庁 = "9"
    総務省 = "10"
    法務省 = "11"
    外務省 = "12"
    財務省 = "13"
    文部科学省 = "14"
    厚生労働省 = "15"
    農林水産省 = "16"
    経済産業省 = "17"
    国土交通省 = "18"
    環境省 = "19"
    消費者庁 = "20"
    復興庁 = "21"
    公正取引委員会 = "22"
    個人情報保護委員会 = "23"
    特許庁 = "24"
    消防庁 = "25"
    資源エネルギー庁 = "26"
    中小企業庁 = "27"
    情報処理推進機構 = "28"
    製品評価技術基盤機構 = "29"
    国立研究開発法人新エネルギー産業技術総合開発機構 = "30"
    工業所有権情報研修館 = "31"
    中小企業基盤整備機構 = "32"
    石油天然ガス金属鉱物資源機構 = "33"
    日本貿易振興機構 = "34"
    観光庁 = "35"
    気象庁 = "36"
    原子力規制委員会 = "37"
    内閣法制局 = "38"
    水産庁 = "39"
    独立行政法人郵便貯金簡易生命保険管理郵便局ネットワーク支援機構 = "40"
    公益財団法人食品等流通合理化促進機構 = "41"
    海上保安庁 = "42"
    スポーツ庁 = "43"
    警察庁 = "44"
    デジタル庁 = "45"
    林野庁 = "46"
    文化庁 = "47"
    公害等調整委員会 = "48"
    こども家庭庁 = "49"



"""HTTP ヘルパーテスト。"""

from __future__ import annotations

import time

import httpx
import pytest

from gbizinfo.http import (
    AsyncRateLimiter,
    SyncRateLimiter,
    build_request_headers,
    build_user_agent,
    compute_wait_time,
    full_jitter_backoff,
    parse_retry_after,
    should_retry_http_status,
    should_retry_transport_error,
)


class TestSyncRateLimiter:
    def test_acquire_enforces_interval(self):
        limiter = SyncRateLimiter(rate_limit_per_sec=10.0)  # 100ms interval
        limiter.acquire()
        start = time.monotonic()
        limiter.acquire()
        elapsed = time.monotonic() - start
        assert elapsed >= 0.05  # at least some wait

    def test_zero_rate_no_wait(self):
        limiter = SyncRateLimiter(rate_limit_per_sec=0.0)
        assert limiter.acquire() == 0.0


@pytest.mark.anyio()
class TestAsyncRateLimiter:
    async def test_acquire(self):
        limiter = AsyncRateLimiter(rate_limit_per_sec=10.0)
        await limiter.acquire()
        await limiter.acquire()  # should wait

    async def test_zero_rate_no_wait(self):
        limiter = AsyncRateLimiter(rate_limit_per_sec=0.0)
        assert await limiter.acquire() == 0.0


class TestShouldRetry:
    def test_retry_on_429(self):
        assert should_retry_http_status(429) is True

    def test_retry_on_500(self):
        assert should_retry_http_status(500) is True

    def test_retry_on_502(self):
        assert should_retry_http_status(502) is True

    def test_retry_on_503(self):
        assert should_retry_http_status(503) is True

    def test_retry_on_504(self):
        assert should_retry_http_status(504) is True

    def test_no_retry_on_400(self):
        assert should_retry_http_status(400) is False

    def test_no_retry_on_404(self):
        assert should_retry_http_status(404) is False

    def test_transport_timeout(self):
        assert should_retry_transport_error(httpx.ReadTimeout("timeout")) is True

    def test_transport_connect(self):
        assert should_retry_transport_error(httpx.ConnectError("conn")) is True

    def test_transport_value_error(self):
        assert should_retry_transport_error(ValueError("nope")) is False


class TestParseRetryAfter:
    def test_numeric(self):
        response = httpx.Response(429, headers={"Retry-After": "5"})
        assert parse_retry_after(response) == 5.0

    def test_missing(self):
        response = httpx.Response(429)
        assert parse_retry_after(response) is None


class TestComputeWaitTime:
    def test_retry_after_takes_priority(self):
        wait = compute_wait_time(attempt=0, base=0.5, cap=8.0, retry_after=10.0)
        assert wait == 10.0

    def test_backoff_when_no_retry_after(self):
        wait = compute_wait_time(attempt=0, base=0.5, cap=8.0, retry_after=None)
        assert 0 <= wait <= 0.5


class TestFullJitterBackoff:
    def test_bounded(self):
        for _ in range(100):
            val = full_jitter_backoff(attempt=3, base=0.5, cap=8.0)
            assert 0 <= val <= 4.0  # min(8.0, 0.5 * 2^3) = 4.0


class TestBuildUserAgent:
    def test_format(self):
        ua = build_user_agent(version="0.1.0")
        assert ua.startswith("gbizinfo/0.1.0 python/")


class TestBuildRequestHeaders:
    def test_contains_token(self):
        headers = build_request_headers(user_agent="test/1.0", api_token="my-token")
        assert headers["X-hojinInfo-api-token"] == "my-token"
        assert headers["Accept"] == "application/json"
        assert headers["User-Agent"] == "test/1.0"



"""契約テスト - fixtures の JSON スナップショットに対するパース互換性。"""

from __future__ import annotations

import json
from pathlib import Path

from gbizinfo.models._generated import (
    HojinInfoResponseSearchV2,
    HojinInfoResponseV2,
    HojinInfoUpdateInfoResponseV2,
)
from gbizinfo.models.responses import HojinInfo, HojinInfoSearch

FIXTURES_DIR = Path(__file__).parent / "fixtures"


class TestSearchResponseContract:
    """search_response.json のパース。"""

    def test_parse(self):
        data = json.loads((FIXTURES_DIR / "search_response.json").read_text(encoding="utf-8"))
        resp = HojinInfoResponseSearchV2.model_validate(data)
        assert resp.hojin_infos is not None
        assert len(resp.hojin_infos) == 2
        assert resp.hojin_infos[0].name == "テスト株式会社"
        assert resp.id == "test-id-001"

    def test_items_as_search_model(self):
        data = json.loads((FIXTURES_DIR / "search_response.json").read_text(encoding="utf-8"))
        resp = HojinInfoResponseSearchV2.model_validate(data)
        for h in resp.hojin_infos or []:
            item = HojinInfoSearch.model_validate(h.model_dump(by_alias=True))
            assert item.corporate_number is not None


class TestDetailResponseContract:
    """hojin_detail_response.json のパース。"""

    def test_parse(self):
        data = json.loads((FIXTURES_DIR / "hojin_detail_response.json").read_text(encoding="utf-8"))
        resp = HojinInfoResponseV2.model_validate(data)
        assert resp.hojin_infos is not None
        assert len(resp.hojin_infos) == 1
        info = HojinInfo.model_validate(resp.hojin_infos[0].model_dump(by_alias=True))
        assert info.name == "トヨタ自動車株式会社"
        assert info.capital_stock == 635401790000

    def test_to_flat_dict(self):
        data = json.loads((FIXTURES_DIR / "hojin_detail_response.json").read_text(encoding="utf-8"))
        resp = HojinInfoResponseV2.model_validate(data)
        info = HojinInfo.model_validate(resp.hojin_infos[0].model_dump(by_alias=True))
        flat = info.to_flat_dict()
        assert flat["name"] == "トヨタ自動車株式会社"


class TestUpdateInfoResponseContract:
    """update_info_response.json のパース。"""

    def test_parse(self):
        data = json.loads((FIXTURES_DIR / "update_info_response.json").read_text(encoding="utf-8"))
        resp = HojinInfoUpdateInfoResponseV2.model_validate(data)
        assert resp.hojin_infos is not None
        assert len(resp.hojin_infos) == 1
        assert resp.totalCount == "150"
        assert resp.totalPage == "2"
        assert resp.pageNumber == "1"


class TestForwardCompatibility:
    """API フィールド追加でパースが壊れないこと。"""

    def test_extra_fields_ignored(self):
        data = {
            "hojin-infos": [
                {
                    "corporate_number": "1234567890123",
                    "name": "テスト",
                    "new_future_field": "should not break",
                    "another_new_field": [1, 2, 3],
                }
            ],
            "id": "test",
            "message": "ok",
            "brand_new_metadata": {"key": "value"},
        }
        resp = HojinInfoResponseSearchV2.model_validate(data)
        assert resp.hojin_infos is not None
        assert resp.hojin_infos[0].name == "テスト"



"""モデルテスト。"""

from __future__ import annotations

import json

from gbizinfo.models.responses import HojinInfo, HojinInfoSearch, SearchResult, UpdateResult


class TestToFlatDict:
    """to_flat_dict() テスト。"""

    def test_count_strategy(self):
        info = HojinInfo.model_validate({
            "name": "テスト株式会社",
            "corporate_number": "1234567890123",
            "certification": [
                {"title": "認定A"},
                {"title": "認定B"},
            ],
        })
        flat = info.to_flat_dict(lists="count")
        assert flat["name"] == "テスト株式会社"
        assert flat["certification_count"] == 2

    def test_first_strategy(self):
        info = HojinInfo.model_validate({
            "name": "テスト",
            "corporate_number": "1234567890123",
            "patent": [
                {"patent_type": "特許", "title": "特許A"},
                {"patent_type": "意匠", "title": "意匠B"},
            ],
        })
        flat = info.to_flat_dict(lists="first")
        assert flat["patent_patent_type"] == "特許"

    def test_json_strategy(self):
        info = HojinInfo.model_validate({
            "name": "テスト",
            "corporate_number": "1234567890123",
            "procurement": [
                {"title": "調達A"},
            ],
        })
        flat = info.to_flat_dict(lists="json")
        parsed = json.loads(flat["procurement"])
        assert len(parsed) == 1

    def test_explode_strategy(self):
        info = HojinInfo.model_validate({
            "name": "テスト",
            "corporate_number": "1234567890123",
            "subsidy": [
                {"title": "補助金A"},
                {"title": "補助金B"},
            ],
        })
        flat = info.to_flat_dict(lists="explode", max_items=10)
        assert flat["subsidy_0_title"] == "補助金A"
        assert flat["subsidy_1_title"] == "補助金B"


class TestAlias:
    """ハイフン alias テスト。"""

    def test_hojin_infos_alias(self):
        from gbizinfo.models._generated import HojinInfoResponseSearchV2
        resp = HojinInfoResponseSearchV2.model_validate({
            "hojin-infos": [{"name": "テスト", "corporate_number": "1234567890123"}],
            "id": "test",
        })
        assert resp.hojin_infos is not None
        assert len(resp.hojin_infos) == 1

    def test_populate_by_name(self):
        from gbizinfo.models._generated import HojinInfoResponseSearchV2
        resp = HojinInfoResponseSearchV2.model_validate({
            "hojin_infos": [{"name": "テスト", "corporate_number": "1234567890123"}],
            "id": "test",
        })
        assert resp.hojin_infos is not None


class TestExtraIgnore:
    """extra="ignore" テスト。"""

    def test_unknown_field_ignored(self):
        info = HojinInfo.model_validate({
            "name": "テスト",
            "corporate_number": "1234567890123",
            "unknown_future_field": "should be ignored",
            "another_field": 42,
        })
        assert info.name == "テスト"


class TestSearchResult:
    def test_to_flat_dicts(self):
        result = SearchResult([
            HojinInfoSearch.model_validate({"name": "A", "corporate_number": "1234567890123"}),
            HojinInfoSearch.model_validate({"name": "B", "corporate_number": "9876543210123"}),
        ])
        dicts = result.to_flat_dicts()
        assert len(dicts) == 2


class TestUpdateResult:
    def test_metadata(self):
        result = UpdateResult(
            items=[HojinInfo.model_validate({"name": "A", "corporate_number": "1234567890123"})],
            total_count=100,
            total_page=2,
            page_number=1,
        )
        assert result.total_count == 100
        assert result.total_page == 2
